/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.example.android.vinylfinder;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.util.SparseBooleanArray;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * Fragmento para exibir os resultados deletados diretamente pelo usuário, ou pela utilização de filtros.
 */
public class DeletadosFragment extends Fragment {
    //public static boolean intentAberto = false;

    public ProgressBar loadingIndicatorDeletados;
    public TextView textViewLoadingIndicatorDeletados;
    static DeletadosFragment df;

    //Adapter para a lista de discos deletados
    DeletadosRecordAdapter mDeletadosAdapter;

    final static List<Record> recordsDeletadosGlobal = new ArrayList<>();

    static public View rootView;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
        df = DeletadosFragment.this;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if (item.isChecked())
            item.setChecked(false);
        else
            item.setChecked(true);

        return true;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.fragment_deletados, container, false);

        // Esconder teclado, loading indicator e textView porque os dados foram carregados
        esconderTeclado(getActivity());
        esconderLoadingIndicator();
        esconderTextViewDeAndamentoDoLoadingIndicator();

        configurarListView();
        setHasOptionsMenu(true);

        return rootView;
    }

    //Realiza a pré-configuração da ListView, criando listeners e associando campos com os ids presentes em Resources
    private void configurarListView() {
        setarTextoASerExibidoParaListViewVazia((ListView) rootView.findViewById(R.id.listview_record_deletados));
        setarAdaptadorParaListView((ListView) rootView.findViewById(R.id.listview_record_deletados));
        setarModoDeEscolhaParaDelecoesMultiplasNaListView((ListView) rootView.findViewById(R.id.listview_record_deletados));
        setarClickListenerParaListView((ListView) rootView.findViewById(R.id.listview_record_deletados));
    }

    private void setarModoDeEscolhaParaDelecoesMultiplasNaListView(final ListView recordsDeletadosListView) {
        recordsDeletadosListView.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE_MODAL);
        recordsDeletadosListView.setMultiChoiceModeListener(new AbsListView.MultiChoiceModeListener() {

            @Override
            public boolean onPrepareActionMode(android.view.ActionMode mode, Menu menu) {
                // TODO  Auto-generated method stub
                return false;
            }


            @Override
            public void onDestroyActionMode(android.view.ActionMode mode) {
                // TODO  Auto-generated method stub

            }

            @Override
            public boolean onCreateActionMode(android.view.ActionMode mode, Menu menu) {
                // TODO  Auto-generated method stub
                mode.getMenuInflater().inflate(R.menu.menu_deletados_contextual, menu);
                return true;

            }

            @Override
            public boolean onActionItemClicked(final android.view.ActionMode mode,
                                               MenuItem item) {
                switch (item.getItemId()) {
                    case R.id.selectAll:
                        removerItensPossivelmenteJaSelecionados();
                        selecionarTodosOsItemsDaListView(recordsDeletadosListView);
                        alterarOTituloDaActionBarParaRefletirOTotalDeItensSelecionados(mode);
                        return true;
                    case R.id.reverter:
                        AlertDialog.Builder builder = configurarAlterDialogDeConfirmacaoDeReciclagemDeItem(mode);
                        exibirAlertDialog(builder);
                        return true;
                    default:
                        return false;
                }
            }


            @Override
            public void onItemCheckedStateChanged(android.view.ActionMode mode,
                                                  int position, long id, boolean checked) {
                // TODO  Auto-generated method stub
                final int checkedCount = recordsDeletadosListView.getCheckedItemCount();
                mode.setTitle(checkedCount + "  Selecionado(s)");
                mDeletadosAdapter.toggleSelection(position);
            }
        });
    }

    private void alterarOTituloDaActionBarParaRefletirOTotalDeItensSelecionados(android.view.ActionMode mode) {
        mode.setTitle(recordsDeletadosGlobal.size() + "  Selecionados");
    }

    private void setarClickListenerParaListView(ListView recordsDeletadosListView) {
        recordsDeletadosListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long l) {
                // Achar o record atual que foi clicado
                Record recordClicadoPeloUsuario = mDeletadosAdapter.getItem(position);

                // Converte o URL String em um objeto URI (para passar no construtor de Intent)
                Uri recordUri = Uri.parse(recordClicadoPeloUsuario.getmPermalink());

                Intent intent = new Intent(Intent.ACTION_VIEW, recordUri);
                intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                startActivity(intent);
                //intentAberto = true;
                //System.out.println(intentAberto);
            }
        });
    }

    //Configuração do Adapter, através da DeletadosRecordAdapter
    private void setarAdaptadorParaListView(final ListView recordsDeletadosListView) {
        mDeletadosAdapter = new DeletadosRecordAdapter(getActivity(), new ArrayList<Record>());
        recordsDeletadosListView.setAdapter(mDeletadosAdapter);

    }

    private void exibirAlertDialog(AlertDialog.Builder builder) {
        AlertDialog alert = builder.create();
        alert.setTitle("Confirmation"); // dialog  Title
        alert.show();
    }

    private AlertDialog.Builder configurarAlterDialogDeConfirmacaoDeReciclagemDeItem(final android.view.ActionMode mode) {
        AlertDialog.Builder builder = new AlertDialog.Builder(
                getContext());
        builder.setMessage("Confirmar reciclagem do(s) disco(s) selecionado(s)?");

        configurarBotaoNegativo(builder);
        configurarBotaoPositivo(builder, mode);

        return builder;
    }

    private void configurarBotaoPositivo(AlertDialog.Builder builder, final android.view.ActionMode mode) {
        builder.setPositiveButton("Sim", new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                SparseBooleanArray selected = mDeletadosAdapter
                        .getSelectedIds();
                for (int i = (selected.size() - 1); i >= 0; i--) {
                    if (selected.valueAt(i)) {
                        Record itemSelecionado = mDeletadosAdapter
                                .getItem(selected.keyAt(i));
                        // Remove os Records selecionados da aba Deletados
                        removerItemDeDeletadosFragment(itemSelecionado);
                        // Adiciona os Records selecionados na aba Resultados
                        adicionarItemEmResultadosFragment(itemSelecionado);
                        //Realiza novo cálculo de média e insere na aba Estatísticas
                        configurarTextViewMediaFinal(realizarSomaDosPrecos());
                        //configurarTextViewSomaDosRecordsDesvioPadraoECoeficienteDeVariacao(realizarSomaDosPrecos());
                    }
                }
                mode.finish();
                selected.clear();
                notificarMudancasNoAdaptadorEmDeletadosFragment();
                notificarMudancasNoAdaptadorEmResultadosFragment();

            }
        });
    }

    private void notificarMudancasNoAdaptadorEmDeletadosFragment() {
        mDeletadosAdapter.notifyDataSetChanged();
        mDeletadosAdapter.notifyDataSetInvalidated();
    }

    private void notificarMudancasNoAdaptadorEmResultadosFragment() {
        ResultadosFragment.rf.mAdapter.notifyDataSetChanged();
        ResultadosFragment.rf.mAdapter.notifyDataSetInvalidated();
    }

    /**private void configurarTextViewSomaDosRecordsDesvioPadraoECoeficienteDeVariacao(Double somaDosPrecos) {
        TextView totalDeRegistros = (TextView) EstatisticasFragment.rootView.findViewById(R.id.textViewTotalRegistros);
        int somaDosRecords = ResultadosFragment.recordsGlobal.size();
        String somaDosRecordsEmString = Integer.toString(somaDosRecords);
        Double desvioPadrao = EstatisticasFragment.calcularDesvioPadrao(calcularMediaEmDouble(somaDosPrecos), ResultadosFragment.recordsGlobal);
        Double coeficienteVariacao = EstatisticasFragment.calcularcoeficienteDeVariacaoDosRecords(calcularMediaEmDouble(somaDosPrecos), desvioPadrao);

        configurarTextoDaTextView(totalDeRegistros, somaDosRecordsEmString, desvioPadrao, coeficienteVariacao);
    }**/

    private void configurarTextoDaTextView(TextView totalDeRegistros, String somaDosRecordsEmString, Double desvioPadrao, Double coeficienteVariacao) {
        totalDeRegistros.setText("Total de Registros: " + somaDosRecordsEmString + "\nDesvio Padrão: " + desvioPadrao + "\nC.V: " + coeficienteVariacao);
        totalDeRegistros.setTextSize(10);
        totalDeRegistros.setTextColor(ContextCompat.getColor(EstatisticasFragment.rootView.getContext(), R.color.colorPrice));
    }

    private void configurarTextViewMediaFinal(Double somaDosPrecos) {
        Double mediaFinalEmDouble = calcularMediaFinalERetornarEmDouble(somaDosPrecos);
        TextView mediaFinal = (TextView) EstatisticasFragment.rootView.findViewById(R.id.textViewMedia);
        configurarTextoDaTextViewMediaFinal(mediaFinalEmDouble, mediaFinal);
        mediaFinal.setVisibility(View.VISIBLE);
    }

    private void configurarTextoDaTextViewMediaFinal(Double mediaFinalEmDouble, TextView mediaFinal) {
        Activity activity = (Activity) EstatisticasFragment.rootView.getContext();
        mediaFinal.setText(converterDoubleEmReais(mediaFinalEmDouble));
        mediaFinal.setTextSize(50);
        mediaFinal.setTextColor(ContextCompat.getColor(activity, R.color.colorPrice));
    }

    public String converterDoubleEmReais(Double d) {
        Locale ptBr = new Locale("pt", "BR");
        return NumberFormat.getCurrencyInstance(ptBr).format(d);
    }

    private Double calcularMediaFinalERetornarEmDouble(Double somaDosPrecos) {
       return calcularMediaEmDouble(somaDosPrecos);
    }

    private Double calcularMediaEmDouble(Double somaDosPrecos) {
        return EstatisticasFragment.fazerMedia(somaDosPrecos, ResultadosFragment.recordsGlobal.size());
    }

    private Double realizarSomaDosPrecos() {
        Double somaDosPrecos = 0.0;
        for (int j = 0; j < ResultadosFragment.recordsGlobal.size(); j++) {
            somaDosPrecos += ResultadosFragment.recordsGlobal.get(j).getmPrice();
        }
        return somaDosPrecos;
    }

    private void adicionarItemEmResultadosFragment(Record selecteditem) {
        ResultadosFragment.rf.mAdapter.add(selecteditem);
        ResultadosFragment.recordsGlobal.add(selecteditem);
    }

    private void removerItemDeDeletadosFragment(Record selecteditem) {
        mDeletadosAdapter.remove(selecteditem);
        recordsDeletadosGlobal.remove(selecteditem);
    }

    private void configurarBotaoNegativo(AlertDialog.Builder builder) {
        builder.setNegativeButton("Não", new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                // TODO  Auto-generated method stub

            }
        });
    }

    private void removerItensPossivelmenteJaSelecionados() {
        mDeletadosAdapter.removeSelection();
    }

    private void selecionarTodosOsItemsDaListView(ListView recordsDeletadosListView) {
        for (int i = 0; i < recordsDeletadosGlobal.size(); i++) {
            recordsDeletadosListView.setItemChecked(i, true);
            //  listviewadapter.toggleSelection(i);
        }
    }

    private void setarTextoASerExibidoParaListViewVazia(ListView recordsDeletadosListView) {
        TextView mEmptyStateTextView = (TextView) rootView.findViewById(R.id.empty_view_deletados);
        recordsDeletadosListView.setEmptyView(mEmptyStateTextView);
    }

    private void esconderTextViewDeAndamentoDoLoadingIndicator() {
        textViewLoadingIndicatorDeletados = (TextView) rootView.findViewById(R.id.textView_loading_indicator_deletados);
        textViewLoadingIndicatorDeletados.setVisibility(View.GONE);
    }

    private void esconderLoadingIndicator() {
        loadingIndicatorDeletados = (ProgressBar) rootView.findViewById(R.id.loading_indicator_deletados);
        loadingIndicatorDeletados.setVisibility(View.GONE);
    }

    private void esconderTeclado(Activity activity) {
        InputMethodManager inputMethodManager =
                (InputMethodManager) activity.getSystemService(
                        Activity.INPUT_METHOD_SERVICE);
        inputMethodManager.hideSoftInputFromWindow(
                new View(activity).getWindowToken(), 0);
    }
}