/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.example.android.vinylfinder;

import android.Manifest;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.app.Fragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.res.ResourcesCompat;
import android.support.v4.view.MenuItemCompat;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.SearchView;
import android.text.Html;
import android.text.InputType;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;

import java.io.IOException;
import java.text.Normalizer;
import java.text.NumberFormat;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import com.koushikdutta.urlimageviewhelper.UrlImageViewHelper;

import br.com.jansenfelipe.androidmask.MaskEditTextChangedListener;

import static com.example.android.vinylfinder.ResultadosFragment.rf;
import static com.example.android.vinylfinder.ResultadosFragment.rootView;

/**
 * Displays a {@link ViewPager} where each page shows a different day of the week.
 */
public class MainActivity extends AppCompatActivity implements
        TabHost.OnTabChangeListener, ViewPager.OnPageChangeListener, android.support.v4.app.LoaderManager.LoaderCallbacks<List<Record>> {

    private TabHost mTabHost;
    private HashMap<String, TabInfo> mapTabInfo = new HashMap<String, TabInfo>();
    private ViewPager viewPager;
    private static Menu mOptionsMenu;
    public static MainActivity mn;
    private static String searchQueryGlobal;
    private LocationManager locationManager;
    private static String searchAlbum;
    private static String searchBand;
    private String cepUsuarioLocal;
    public CheckBox dontShowAgain;
    public static final String PREFS_NAME = "MyPrefsFile1";
    public static Boolean isRunning = false;
    public boolean cepRetornouNull;

    private final int ERROR_CODE_IMAGE_NOT_FOUND = -1;

    private String cepUsuarioLocalComHifen;
    int currentTab;
    boolean primeiraInflada;
    boolean chekItemUsado;
    boolean chekItemNovo;
    boolean chekItemLongPlay;
    boolean chekItemCompacto;
    boolean chekItemNacional;
    boolean chekItemImportado;

    private static boolean opcaoNovoSelecionadaNoFiltroAntesDaPesquisa;
    private static boolean opcaoUsadoSelecionadaNoFiltroAntesDaPesquisa;
    private static boolean opcaoCompactoSelecionadaNoFiltroAntesDaPesquisa;
    private static boolean opcaoLongPlaySelecionadaNoFiltroAntesDaPesquisa;
    private static boolean opcaoNacionalSelecionadaNoFiltroAntesDaPesquisa;
    private static boolean opcaoImportadoSelecionadaNoFiltroAntesDaPesquisa;

    boolean checkBoxSelecionada;

    ListView recordListView;

    /**
     * URL para realizar busca no banco de dados doMercadoLivre por informações de discos
     */
    private static final String MERCADO_LIVRE_BASE_SEARCH_URL =
            "https://api.mercadolibre.com/sites/MLB/search";
    private static final boolean ERROR_CEP_NULL = true;
    private static final boolean RESET_ERROR_VARIABLE = false;

    // INÍCIO - Variáveis e Métodos relacionados as TABs

    // Informações Gerais da Tab
    private class TabInfo {
        private String tag;
        private Class<?> clss;
        private Bundle args;
        private Fragment fragment;

        TabInfo(String tag, Class<?> clazz, Bundle args) {
            this.tag = tag;
            this.clss = clazz;
            this.args = args;
        }
    }

    // Inicialização da TabHost, inflando as abas de "Estatísticas", "Resultados" e "Deletados", através da chamada de seus fragmentos
    private void initialiseTabHost(Bundle args) {

        LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View resultadosTab = inflater.inflate(R.layout.resultados_tab, null);

        mTabHost = (TabHost) findViewById(android.R.id.tabhost);
        mTabHost.setup();
        TabInfo tabInfo;
        MainActivity.AddTab(
                this,
                this.mTabHost,
                this.mTabHost
                        .newTabSpec("Estatísticas")
                        .setIndicator(null, ResourcesCompat.getDrawable(getResources(), R.drawable.ic_insert_chart_black_24dp, null)),
                (tabInfo = new TabInfo("Estatísticas", EstatisticasFragment.class, args)));
        this.mapTabInfo.put(tabInfo.tag, tabInfo);
        //mTabHost.getTabWidget().getChildAt(0).setBackgroundColor(Color.RED);
        MainActivity.AddTab(this,
                this.mTabHost,
                this.mTabHost
                        .newTabSpec("Resultados")
                        .setIndicator(null, ResourcesCompat.getDrawable(getResources(), R.drawable.ic_format_list_bulleted_black_24dp, null)),
                (tabInfo = new TabInfo("Resultados", ResultadosFragment.class, args)));
        this.mapTabInfo.put(tabInfo.tag, tabInfo);
        MainActivity.AddTab(
                this,
                this.mTabHost,
                this.mTabHost
                        .newTabSpec("Deletados")
                        .setIndicator(null, ResourcesCompat.getDrawable(getResources(), R.drawable.ic_delete_sweep_black_24dp, null)),
                (tabInfo = new TabInfo("Deletados", DeletadosFragment.class, args)));
        this.mapTabInfo.put(tabInfo.tag, tabInfo);

        mTabHost.setOnTabChangedListener(this);
    }

    // Um simples factory que retorna a View para o TabHost
    class TabFactory implements TabHost.TabContentFactory {

        private final Context mContext;

        public TabFactory(Context context) {
            mContext = context;
        }

        public View createTabContent(String tag) {
            View v = new View(mContext);
            v.setMinimumWidth(0);
            v.setMinimumHeight(0);
            return v;
        }

    }


    private static void AddTab(MainActivity activity, TabHost tabHost,
                               TabHost.TabSpec tabSpec, TabInfo tabInfo) {
        tabSpec.setContent(activity.new TabFactory(activity));
        tabHost.addTab(tabSpec);

    }

    public void onTabChanged(String tag) {
        // Avisa para o mViewPager qual a Tab que está ativa
        viewPager.setCurrentItem(mTabHost.getCurrentTab());
    }

    // FINAL - Variáveis e Métodos relacionados as TABs

    // INÍCIO - Overriden methods - INÍCIO

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        mn = MainActivity.this;
        primeiraInflada = true;

        //Pesquisa CEP, em arquivo de Shared Preferences, armazenado
        SharedPreferences CEPsArmazenadosSharedPreferences = getSharedPreferences(
                "CEPsArmazenados", Context.MODE_PRIVATE);
        //System.out.println(CEPsArmazenadosSharedPreferences);
        final String ultimoCEP = CEPsArmazenadosSharedPreferences.getString("ultimoCEP", null);
        //Verifica se a flag de Não Exibir Cep foi selecionada previamente
        SharedPreferences NaoExibirMaisPopupDeCepSharedPreferences = getSharedPreferences(
                "NaoExibirMaisPopupDeCep", Context.MODE_PRIVATE);

        //System.out.println(NaoExibirMaisPopupDeCepSharedPreferences);
        final String opcaoSelecionadaSobreExibirPopupDeCEP = NaoExibirMaisPopupDeCepSharedPreferences.getString("modoSelecionado", null);

        //Se não houver CEP armazenado em SharedPreferences, e não tiver sido feita opção de
        //não exibir a popup, exiba a popup de escolher entre inserir CEP ou usar GPS
        if (ultimoCEP == null && opcaoSelecionadaSobreExibirPopupDeCEP == null) {
            dialogCEP();
        //Se já houver um CEP armazenado em SharedPreferences, e não tiver sido feita opção de
        //não exibir a popup, exiba a popup que mostra o CEP anterior e pede confirmação
        } else if ((ultimoCEP != null && opcaoSelecionadaSobreExibirPopupDeCEP == null) || (ultimoCEP != null && opcaoSelecionadaSobreExibirPopupDeCEP.equals("NOVO CEP INSERIDO"))){
            dialogCEPJaExistente(ultimoCEP);
        //Se foi feita opção de não exibir mais a popup, não exiba-a
        } else if (ultimoCEP != null && opcaoSelecionadaSobreExibirPopupDeCEP.equals("NÃO EXIBIR NOVAMENTE")){
            ResultadosFragment.cepUsuario = ultimoCEP;
        }

        this.initialiseTabHost(savedInstanceState);
        if (savedInstanceState != null) {
            // Define a Tab de acordo com o estado salvo
            mTabHost.setCurrentTabByTag(savedInstanceState.getString("tab"));
        }
        // Inicializa o ViewPager
        this.intialiseViewPager();

    }

    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        super.onCreateOptionsMenu(menu);

        mOptionsMenu = menu;
        //Infla um search bar
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_search_bar, menu);

        //Cria os listeners e popula as variáveis chekItemNovo, chekItemUsado, chekItemNacional, chekItemImportado
        //chekItemLongPlay e chekItemCompacto
        setarListenersParaRaddioButtons(menu);

        if (primeiraInflada) {
            //Marca os check-box de Usado, Nacional e LongPlay, que são os filtros padrões
            setarFiltrosPadroes(menu);
            atualizarVariaveisGlobais(menu);

            primeiraInflada = false;
        } else {
            if (chekItemNovo){
                menu.findItem(R.id.novo).setChecked(true);
            } else if (chekItemUsado){
                menu.findItem(R.id.usado).setChecked(true);
            }

            if (chekItemNacional){
                menu.findItem(R.id.nacional).setChecked(true);
            } else if (chekItemImportado) {
                menu.findItem(R.id.importado).setChecked(true);
            }

            if (chekItemLongPlay){
                menu.findItem(R.id.longplay).setChecked(true);
            } else if (chekItemCompacto){
                menu.findItem(R.id.compacto).setChecked(true);
            }
        }

        //Pega o Componente.
        final SearchView mSearchView = (SearchView) menu.findItem(R.id.search)
                .getActionView();

        //Define um texto de ajuda.
        mSearchView.setQueryHint("Banda - Álbum");

        //Cria um action listener exibir e ocultar a search view com o botão de pesquisa ao lado
        MenuItemCompat.setOnActionExpandListener(menu.findItem(R.id.search), new MenuItemCompat.OnActionExpandListener() {

            @Override
            public boolean onMenuItemActionExpand(final MenuItem item) {
                MainActivity.this.setItemsVisibility(menu, menu.findItem(R.id.search), false);
                return true;
            }

            @Override
            public boolean onMenuItemActionCollapse(final MenuItem item) {
                MainActivity.this.setItemsVisibility(menu, menu.findItem(R.id.search), true);
                return true;
            }
        });

        //Cria um listener para a pesquisa em sí, quando o botão de lupa é clicado no teclado do dispositivo
        mSearchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {

            //Pesquisa inicia aqui, após o termo de busca ser enviado
            @Override
            public boolean onQueryTextSubmit(String query) {
                //Forçar a existência de hífen entre o disco e a banda
                if (!query.contains("-")){
                    alertDialogParaFaltaDeHifenNoTermoDeBusca();
                } else {
                    searchQueryGlobal = query;

                    //Ligação entre View e Controller
                    TextView textViewTituloQuery = (TextView) MainActivity.this.findViewById(R.id.textViewTituloQuery);

                    //Limpa a TextView que mostra o resultado por escrito
                    textViewTituloQuery.setText("");

                    //Verifica as opções selecionadas para os filtros pré-pesquisa
                    verificaOpcoesSelecionadasNoMenuDeOpcoes();

                    //Realização da Pesquisa em sí
                    realizarPesquisa();

                    //Ajustes pós pesquisa para re-inserir o ícone de pesquisa e tirar o foco da SearchView
                    mSearchView.setIconified(true);
                    mSearchView.clearFocus();

                    if (menu != null) {
                        (menu.findItem(R.id.search)).collapseActionView();
                    }
                }
                return false;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return false;
            }
        });
        return true;
    }

    //Filtros padrões são Usado, LongPlay e Nacional
    public void setarFiltrosPadroes(Menu menu) {
        menu.findItem(R.id.usado).setChecked(true);
        chekItemUsado = true;
        chekItemNovo = false;
        menu.findItem(R.id.longplay).setChecked(true);
        chekItemLongPlay = true;
        chekItemCompacto = false;
        menu.findItem(R.id.nacional).setChecked(true);
        chekItemNacional = true;
        chekItemImportado = false;
    }

    private AlertDialog alertDialogParaFaltaDeHifenNoTermoDeBusca() {
        return new AlertDialog.Builder(this)
                .setTitle("Erro!")
                .setMessage("É necessário realizar a busca com um hífen entre o nome da banda e o nome do disco")
                .setNegativeButton("OK", null)
                .show();
    }

    //RadioButtons são exibidos na ordem em que são dispostos no código
    private void setarListenersParaRaddioButtons(Menu menu) {
        menu.findItem(R.id.usado).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemUsado = true;
                chekItemNovo = false;
                return true;
            }
        });

        menu.findItem(R.id.novo).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemUsado = false;
                chekItemNovo = true;
                return true;
            }
        });

        menu.findItem(R.id.longplay).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemLongPlay = true;
                chekItemCompacto = false;
                return true;
            }
        });

        menu.findItem(R.id.compacto).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemLongPlay = false;
                chekItemCompacto = true;
                return true;
            }
        });

        menu.findItem(R.id.nacional).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemNacional = true;
                chekItemImportado = false;
                return true;
            }
        });

        menu.findItem(R.id.importado).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

            @Override
            public boolean onMenuItemClick(MenuItem item) {
                item.setChecked(true);
                chekItemNacional = false;
                chekItemImportado = true;
                return true;
            }
        });
    }

    //Verifica os valores selecionados pelo usuário para os filtros pré-pesquisa, e atualiza as variáveis de acordo
    private void atualizarVariaveisGlobais(Menu menu) {
            if (menu.findItem(R.id.novo).isChecked()){
                chekItemNovo = true;
            } else {
                chekItemNovo = false;
            }

            if (menu.findItem(R.id.usado).isChecked()){
                chekItemUsado = true;
            } else {
                chekItemUsado = false;
            }

            if (menu.findItem(R.id.longplay).isChecked()){
                chekItemLongPlay = true;
            } else {
                chekItemLongPlay = false;
            }

            if (menu.findItem(R.id.compacto).isChecked()){
                chekItemCompacto = true;
            } else {
                chekItemCompacto = false;
            }

            if (menu.findItem(R.id.nacional).isChecked()){
                chekItemNacional = true;
            } else {
                chekItemNacional = false;
            }

            if (menu.findItem(R.id.importado).isChecked()){
                chekItemImportado = true;
            } else {
                chekItemImportado = false;
            }
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        Log.d("MainActivity", "On windows focus changed. (" + hasFocus + ")");
System.out.println("\n************************\n" + getLoaderManager().getLoader(0));
System.out.println(isRunning + "\n************************\n");
        isRunning = getLoaderManager().getLoader(0) != null;

        /**if (!hasFocus){
            ResultadosFragment.intentAberto = true;
        }**/
    }
    private void setItemsVisibility(Menu menu, MenuItem exception, boolean visible) {
        for (int i=0; i<menu.size(); ++i) {
            MenuItem item = menu.getItem(i);
            if (item != exception) item.setVisible(visible);
            if (!visible){
                if (i == 1){
                    item.setVisible(true);
                }
            }
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.search:
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onPageSelected(int position) {
        this.mTabHost.setCurrentTab(position);
    }

    @Override
    public void onPageScrolled(int position, float positionOffset,
                               int positionOffsetPixels) {
    }

    @Override
    public void onPageScrollStateChanged(int state) {

    }

    @Override
    public android.support.v4.content.Loader<List<Record>> onCreateLoader(int i, Bundle bundle) {

        //Utilização de Uri para poder realizar append de parâmetros em cima de uma URL base, do Mercado Livre
        Uri baseUri = Uri.parse(MERCADO_LIVRE_BASE_SEARCH_URL);
        Uri.Builder uriBuilder = baseUri.buildUpon();

        //Dissociação da banda e do álbum, presentes no termo de busca
        String[] trimmedSearchQueryGlobal = searchQueryGlobal.split("-");
        searchBand = trimmedSearchQueryGlobal[0];
        searchAlbum = trimmedSearchQueryGlobal[1];

        //Adição de parâmetos que limitam cada pesquisa à 50 resultados por página do JSON
        //retornado pelo Mercado Livre (É o valor máximo permitido pela API do ML)
        uriBuilder.appendQueryParameter("limit", "50");

        //Categoria "Música, Filmes e Seriados", do ML
        uriBuilder.appendQueryParameter("category", "MLB1168");

        //Append de termo de pesquisa para focar em disco de vinil
        if (!searchAlbum.isEmpty() || !searchBand.isEmpty()) {
            if (isCompactoSelected() && isNacionalSelected()) {
                uriBuilder.appendQueryParameter("q", searchAlbum + "+" + searchBand + "+VINIL+LP");
            } else if (isCompactoSelected() && isImportadoSelected()) {
                uriBuilder.appendQueryParameter("q", searchAlbum + "+" + searchBand + "+VINIL+LP");
            } else if (isLongPlaySelected() && isNacionalSelected()) {
                uriBuilder.appendQueryParameter("q", searchAlbum + "+" + searchBand + "+VINIL+LP");
            } else if (isLongPlaySelected() && isImportadoSelected()) {
                uriBuilder.appendQueryParameter("q", searchAlbum + "+" + searchBand + "+VINIL+LP");
            }
        } else {
            //Pesquisa genérica para um último caso, onde o termo de busca foi inserido de maneira incorreta
            if (isCompactoSelected() && isNacionalSelected()) {
                uriBuilder.appendQueryParameter("q", "VINIL+LP");
            } else if (isCompactoSelected() && isImportadoSelected()) {
                uriBuilder.appendQueryParameter("q", "VINIL+LP");
            } else if (isLongPlaySelected() && isNacionalSelected()) {
                uriBuilder.appendQueryParameter("q", "VINIL+LP");
            } else if (isLongPlaySelected() && isImportadoSelected()) {
                uriBuilder.appendQueryParameter("q", "VINIL+LP");
            }
        }
        //Último parâmetro da URL do Mercado Livre, para solicitar o resultado em formato JSON
        uriBuilder.appendQueryParameter("", "#json");

        //Log.v("URL COMPLETA = ", uriBuilder.toString());

        //Chama RecordLoader.java, passando a URL, do Mercado Livre, recém criada
        return new RecordLoader(MainActivity.this, uriBuilder.toString());
    }

    @Override
    public void onLoaderReset(android.support.v4.content.Loader<List<Record>> loader) {
        ListView recordListView = (ListView) rootView.findViewById(R.id.listview_record);
        recordListView.invalidateViews();
        ResultadosFragment.rf.mAdapter.clear();
        ResultadosFragment.rf.mAdapter.notifyDataSetChanged();
    }

    // INÍCIO - OnLoadFinished()
    @Override
    public void onLoadFinished(android.support.v4.content.Loader<List<Record>> loader, List<Record> recordsLocal) {

        ResultadosFragment.recordsGlobal = recordsLocal;

        // Esconde a progress bar
        esconderLoadingIndicatorEmEstatisticasFragment();
        esconderLoadingIndicatorEmResultadosFragment();
        esconderloadingIndicatorEmDeletadosFragment();

        //ResultadosFragment.rf.mEmptyStateTextView.setText(R.string.no_records);
        invalidarViewsDeRecordListViewEmResultadosFragment();

        //Limpar adapter que pode estar com sujeira de pesquisas anteriores
        limparmAdapterDeDadosAntigosEmResultadosFragment();

        // Update na listView de Records, com os resultados recuperados
        if (recordsLocal != null && !recordsLocal.isEmpty()) {
            //Ordenar os resultados
            sortRecordsLocalPorMenorDistanciaEntreCompradorEVendedor(recordsLocal);
            //Adicionar os records na listView
            adicionarRecordsLocalNomAdapterEmResultadosFragment(recordsLocal);
        }

        // Notify para update dos dados na listView de Records na aba Resultados
        notifyDataSetChangedNomAdapterEmResultadosFragment();

        // Seta mensagem com a quantidade de discos encontrados para o termo de busca, na aba Estatísticas
        setarTextoComDescricaoDoResultadoDaPesquisa();

        //A lista não estando vazia...
        if (isrecordsMaiorQueZero(recordsLocal)) {
            //Configura-se a imageView da aba de Estatísticas, pegando a imagem mais quadrada possível
            configurarImageViewDaImagemPrincipal(recordsLocal);
            //Configura-se a textView com a média dos valores dos anúncios contidos na aba Resultados
            configurarTextViewDaMedia(recordsLocal);
            //Dá-se viabilidade ao Fragmento
            EstatisticasFragment.mFab.setVisibility(View.VISIBLE);

            //configurarInformacoesAdicionaisDeTotalDeRegistrosDesvioPadraoECoeficienteDeVariacao(recordsLocal);

        }

        //retornarFlagDeIntentAbertoParaFalse();

        getLoaderManager().destroyLoader(loader.getId());

    }

    /**private void retornarFlagDeIntentAbertoParaFalse() {
        ResultadosFragment.intentAberto = false;
        DeletadosFragment.intentAberto = false;
    }**/

    public void esconderLoadingIndicatorEmEstatisticasFragment() {
        View loadingIndicatorEstatisticas = MainActivity.this.findViewById(R.id.loading_indicator);
        loadingIndicatorEstatisticas.setVisibility(View.GONE);

        TextView textViewloadingIndicatorEstatisticas = (TextView) MainActivity.this.findViewById(R.id.textView_loading_indicator);
        textViewloadingIndicatorEstatisticas.setVisibility(View.GONE);
    }

    public void esconderLoadingIndicatorEmResultadosFragment() {
        View loadingIndicatorResultados = MainActivity.this.findViewById(R.id.loading_indicator_resultados);
        loadingIndicatorResultados.setVisibility(View.GONE);

        TextView textViewloadingIndicatorResultados = (TextView) MainActivity.this.findViewById(R.id.textView_loading_indicator_resultados);
        textViewloadingIndicatorResultados.setVisibility(View.GONE);
    }

    public void esconderloadingIndicatorEmDeletadosFragment() {
        View loadingIndicatorDeletados = MainActivity.this.findViewById(R.id.loading_indicator_deletados);
        loadingIndicatorDeletados.setVisibility(View.GONE);

        TextView textViewloadingIndicatorDeletados = (TextView) MainActivity.this.findViewById(R.id.textView_loading_indicator_deletados);
        textViewloadingIndicatorDeletados.setVisibility(View.GONE);
    }

    public void invalidarViewsDeRecordListViewEmResultadosFragment() {
        recordListView = (ListView) rootView.findViewById(R.id.listview_record);
        recordListView.invalidateViews();
    }

    public void limparmAdapterDeDadosAntigosEmResultadosFragment() {
        ResultadosFragment.rf.mAdapter.clear();
    }

    public void sortRecordsLocalPorMenorDistanciaEntreCompradorEVendedor(List<Record> recordsLocal) {
        Collections.sort(recordsLocal);
    }

    public void adicionarRecordsLocalNomAdapterEmResultadosFragment(List<Record> recordsLocal) {
        ResultadosFragment.rf.mAdapter.addAll(recordsLocal);
    }

    public void notifyDataSetChangedNomAdapterEmResultadosFragment() {
        ResultadosFragment.rf.mAdapter.notifyDataSetChanged();
    }

    public String sizeDeRecordsGlobalEmResultadosFragmentConvertidoParaString() {
        return Integer.toString(ResultadosFragment.recordsGlobal.size());
    }

    public void setarTextoComDescricaoDoResultadoDaPesquisa() {
        ((TextView) MainActivity.this.findViewById(R.id.textViewTituloQuery)).setText("Encontrados " + ResultadosFragment.recordsGlobal.size() +
                " anúncios para " + "\"" + searchQueryGlobal + "\"");
    }

    public Double somaDosPrecosDeTodosOsResultados(List<Record> recordsLocal) {
        Double priceSum = 0.0;
        for (int j = 0; j < recordsLocal.size(); j++) {
            priceSum += recordsLocal.get(j).getmPrice();
        }
        return priceSum;
    }

    public Double calcularMediaFinalEmDouble(List<Record> recordsLocal) {
        return EstatisticasFragment.fazerMedia(somaDosPrecosDeTodosOsResultados(recordsLocal), ResultadosFragment.recordsGlobal.size());
    }

    public boolean isrecordsMaiorQueZero(List<Record> recordsLocal) {
        return recordsLocal.size() > 0;
    }

    public String retornarLarguraEAltura(List<Record> recordsLocal, int index) {
        return recordsLocal.get(index).getmUrlPictureBigSize();
    }

    public String[] separarLarguraEAlturaEmIndicesDiferentes(String larguraEAlturaNaoSeparadas) {
        return larguraEAlturaNaoSeparadas.split("x");
    }

    public String getLarguraEmString(String[] larguraEAlturaSeparadasEmIndicesDiferentesEmString) {
        return larguraEAlturaSeparadasEmIndicesDiferentesEmString[0];
    }

    public String getAlturaEmString(String[] larguraEAlturaSeparadasEmIndicesDiferentesEmString) {
        return larguraEAlturaSeparadasEmIndicesDiferentesEmString[1];
    }

    public Double calcularRazaoEntreLarguraEAltura(Double largura, Double altura) {
        return (largura / altura);
    }

    public boolean imagemIsEntreRangeRestrito(Double razaoEmDouble) {
        return ((razaoEmDouble > 0.95) && (razaoEmDouble < 1.05));
    }

    public boolean imagemIsEntreRangeMenosRestrito(Double razaoEmDouble) {
        return (razaoEmDouble > 0.90) && (razaoEmDouble < 1.10);
    }

    public void configurarImageViewDaImagemPrincipal(List<Record> recordsLocal) {
        //Procura a imagem mais quadrada possível
        int indexDoRecordEmRecordsLocalQueContemImagemDesejada = percorrerRecordsLocalProcurandoImagemComFormatoQuadrado(recordsLocal);

        //Se nada for encontrado, procura um formato menos quadrado
        if (indexDoRecordEmRecordsLocalQueContemImagemDesejada == ERROR_CODE_IMAGE_NOT_FOUND) {
            indexDoRecordEmRecordsLocalQueContemImagemDesejada = percorrerRecordsLocalProcurandoImagemComFormatoMenosQuadrado(recordsLocal);
            if (indexDoRecordEmRecordsLocalQueContemImagemDesejada == ERROR_CODE_IMAGE_NOT_FOUND){
                setarImagemDesejadaComoImageViewImagemPrincipalDeResultadosFragmentXML(recordsLocal, 0);
            } else {
                setarImagemDesejadaComoImageViewImagemPrincipalDeResultadosFragmentXML(recordsLocal, indexDoRecordEmRecordsLocalQueContemImagemDesejada);
            }
        } else {
            setarImagemDesejadaComoImageViewImagemPrincipalDeResultadosFragmentXML(recordsLocal, indexDoRecordEmRecordsLocalQueContemImagemDesejada);
        }

        //Exibir imagem
        ((ImageView) MainActivity.this.findViewById(R.id.imageViewImagemPrincipal)).setVisibility(View.VISIBLE);
        ((ImageView) MainActivity.this.findViewById(R.id.imageViewImagemPrincipal)).requestLayout();
    }

    public void configurarTextViewDaMedia(List<Record> recordsLocal) {
        Double mediaFinalEmDouble = calcularMediaFinalEmDouble(recordsLocal);

        ((TextView) MainActivity.this.findViewById(R.id.textViewMedia)).setVisibility(View.VISIBLE);
        ((TextView) MainActivity.this.findViewById(R.id.textViewMedia)).setText(converterDoubleEmReais(mediaFinalEmDouble));
        ((TextView) MainActivity.this.findViewById(R.id.textViewMedia)).setTextSize(50);
        ((TextView) MainActivity.this.findViewById(R.id.textViewMedia)).setTextColor(ContextCompat.getColor(MainActivity.this.getApplicationContext(), R.color.colorAccent));

    }

    public String converterDoubleEmReais(Double d) {
        Locale ptBr = new Locale("pt", "BR");
        return NumberFormat.getCurrencyInstance(ptBr).format(d);
    }

    /**public void configurarInformacoesAdicionaisDeTotalDeRegistrosDesvioPadraoECoeficienteDeVariacao(List<Record> recordsLocal) {
        Double desvioPadrao = EstatisticasFragment.calcularDesvioPadrao(calcularMediaFinalEmDouble(recordsLocal), ResultadosFragment.recordsGlobal);
        Double coeficienteDeVariacaoDosRecords = EstatisticasFragment.calcularcoeficienteDeVariacaoDosRecords(calcularMediaFinalEmDouble(recordsLocal), desvioPadrao);

        ((TextView) MainActivity.this.findViewById(R.id.textViewTotalRegistros)).setVisibility(View.VISIBLE);
        ((TextView) MainActivity.this.findViewById(R.id.textViewTotalRegistros)).setText("Total de Registros: " + sizeDeRecordsGlobalEmResultadosFragmentConvertidoParaString() + "\nDesvio Padrão: " + desvioPadrao + "\nC.V: " + coeficienteDeVariacaoDosRecords);
        ((TextView) MainActivity.this.findViewById(R.id.textViewTotalRegistros)).setTextSize(10);
        ((TextView) MainActivity.this.findViewById(R.id.textViewTotalRegistros)).setTextColor(ContextCompat.getColor(MainActivity.this.getApplicationContext(), R.color.colorAccent));

    }**/

    public void setarImagemDesejadaComoImageViewImagemPrincipalDeResultadosFragmentXML(List<Record> recordsLocal, int index) {
        UrlImageViewHelper.setUrlDrawable(((ImageView) MainActivity.this.findViewById(R.id.imageViewImagemPrincipal)), recordsLocal.get(index).getmUrlPictureBig());
    }

    public int percorrerRecordsLocalProcurandoImagemComFormatoQuadrado(List<Record> recordsLocal) {
        Double larguraEmDouble;
        Double alturaEmDouble;
        Double razaoEmDouble;
        String larguraEAlturaNaoSeparadasEmString;
        String[] larguraEAlturaSeparadasEmIndicesDiferentesEmString;
        for (int i = 0; i < recordsLocal.size(); i++) {
            System.out.println(recordsLocal.get(i).getmUrlPictureBigSize());
            larguraEAlturaNaoSeparadasEmString = retornarLarguraEAltura(recordsLocal, i);
            larguraEAlturaSeparadasEmIndicesDiferentesEmString = separarLarguraEAlturaEmIndicesDiferentes(larguraEAlturaNaoSeparadasEmString);
            larguraEmDouble = Double.parseDouble(getLarguraEmString(larguraEAlturaSeparadasEmIndicesDiferentesEmString));
            alturaEmDouble = Double.parseDouble(getAlturaEmString(larguraEAlturaSeparadasEmIndicesDiferentesEmString));
            razaoEmDouble = calcularRazaoEntreLarguraEAltura(larguraEmDouble, alturaEmDouble);

            if (imagemIsEntreRangeRestrito(razaoEmDouble)) {
                System.out.println(recordsLocal.get(i).getmUrlPictureBig());
                return i;
            }
        }
        return ERROR_CODE_IMAGE_NOT_FOUND;
    }


    public int percorrerRecordsLocalProcurandoImagemComFormatoMenosQuadrado(List<Record> recordsLocal) {
        Double larguraEmDouble;
        Double alturaEmDouble;
        Double razaoEmDouble;
        String larguraEAlturaNaoSeparadasEmString;
        String[] larguraEAlturaSeparadasEmIndicesDiferentesEmString;
        for (int i = 0; i < recordsLocal.size(); i++) {
            System.out.println(recordsLocal.get(i).getmUrlPictureBigSize());
            larguraEAlturaNaoSeparadasEmString = retornarLarguraEAltura(recordsLocal, i);
            larguraEAlturaSeparadasEmIndicesDiferentesEmString = separarLarguraEAlturaEmIndicesDiferentes(larguraEAlturaNaoSeparadasEmString);
            larguraEmDouble = Double.parseDouble(getLarguraEmString(larguraEAlturaSeparadasEmIndicesDiferentesEmString));
            alturaEmDouble = Double.parseDouble(getAlturaEmString(larguraEAlturaSeparadasEmIndicesDiferentesEmString));
            razaoEmDouble = calcularRazaoEntreLarguraEAltura(larguraEmDouble, alturaEmDouble);
            if (imagemIsEntreRangeMenosRestrito(razaoEmDouble)) {
                System.out.println(recordsLocal.get(i).getmUrlPictureBig());
                return i;
            }
        }
        return ERROR_CODE_IMAGE_NOT_FOUND;
    }

    // FINAL - OnLoadFinished()
    // FINAL - OnLoadFinished()
    // FINAL - OnLoadFinished()

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        // If we've received a touch notification that the user has touched
        // outside the app, finish the activity.
        if (MotionEvent.ACTION_OUTSIDE == event.getAction()) {
            finish();
            return true;
        }
        // Delegate everything else to Activity.
        return super.onTouchEvent(event);
    }

    // INÍCIO - onBackPressed()
    // INÍCIO - onBackPressed()
    // INÍCIO - onBackPressed()

    @Override
    public void onBackPressed() {
        AlertDialog builder = createAndConfigurePositiveAndNegativeButtonOfAlertDialog();

        setNegativeButtonColor(builder);

        setPositiveButtonColor(builder);

    }

    private void setNegativeButtonColor(AlertDialog builder) {
        builder.getButton(DialogInterface.BUTTON_NEGATIVE).setTextColor(ContextCompat.getColor(this, R.color.colorAccent));
    }

    private void setPositiveButtonColor(AlertDialog builder) {
        builder.getButton(DialogInterface.BUTTON_POSITIVE).setTextColor(ContextCompat.getColor(this, R.color.colorAccent));
    }

    private AlertDialog createAndConfigurePositiveAndNegativeButtonOfAlertDialog() {
        return new AlertDialog.Builder(this)
                .setIcon(R.drawable.ic_exit_to_app_black_24dp)
                .setTitle("Encerrar Vinyl Finder")
                .setMessage("Tem certeza que deseja sair?")
                .setPositiveButton("Sim", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        Intent intent = new Intent(Intent.ACTION_MAIN);
                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                        intent.addCategory(Intent.CATEGORY_HOME);
                        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        startActivity(intent);
                        finish();
                    }

                })
                .setNegativeButton("Não", null)
                .show();
    }

    // FINAL - onBackPressed()

    // FINAL - Overriden methods - FINAL

    //Verifica as opções selecionadas para os filtros pré-pesquisa
    public void verificaOpcoesSelecionadasNoMenuDeOpcoes() {
        opcaoNovoSelecionadaNoFiltroAntesDaPesquisa = isNovoSelected();
        opcaoUsadoSelecionadaNoFiltroAntesDaPesquisa = isUsadoSelected();
        opcaoCompactoSelecionadaNoFiltroAntesDaPesquisa = isCompactoSelected();
        opcaoLongPlaySelecionadaNoFiltroAntesDaPesquisa = isLongPlaySelected();
        opcaoNacionalSelecionadaNoFiltroAntesDaPesquisa = isNacionalSelected();
        opcaoImportadoSelecionadaNoFiltroAntesDaPesquisa = isImportadoSelected();
    }

    private static boolean isCompactoSelected() {
        return mOptionsMenu.findItem(R.id.compacto).isChecked();
    }

    private static boolean isLongPlaySelected() {
        return mOptionsMenu.findItem(R.id.longplay).isChecked();
    }

    private static boolean isNovoSelected() {
        return mOptionsMenu.findItem(R.id.novo).isChecked();
    }

    private static boolean isUsadoSelected() {
        return mOptionsMenu.findItem(R.id.usado).isChecked();
    }

    private static boolean isNacionalSelected() {
        return mOptionsMenu.findItem(R.id.nacional).isChecked();
    }

    private static boolean isImportadoSelected() {
        return mOptionsMenu.findItem(R.id.importado).isChecked();
    }

    public void dialogCEP() {

            locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
            final Dialog dialog = configurarDialog();

            Button GPSbutton = (Button) dialog.findViewById(R.id.GPSButton);
            GPSbutton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    //Realiza a conexão com GPS, com posterior calculo das coordenadas, trazudindo-as em CEP
                    calcularLatitudeELongitueEExibirMensagemDeConfirmacaoDeCep();
                    //Exibe mensagem para confirmação do CEP
                    exibirMensagemConfirmarCep();
                    dialog.dismiss();
                }
            });

            Button CEPbutton = (Button) dialog.findViewById(R.id.CEPButton);
            CEPbutton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    //Exibe mensagem para inserir CEP
                    exibirMensagemInserirCep();
                    dialog.dismiss();

                }
            });

            dialog.show();

        }

    public void dialogCEPJaExistente(final String ultimoCEP) {
        checkBoxSelecionada = false;

        final String ultimoCEPComMascara = ultimoCEP.substring(0, 2) + "." + ultimoCEP.substring(2, 5) + "-" + ultimoCEP.substring(5, ultimoCEP.length());

        final Dialog dialog = configurarDialogCEPJaExistente(ultimoCEP);

        TextView textViewUltimoCepUtilizado = (TextView) dialog.findViewById(R.id.textViewUltimoCEP);
        textViewUltimoCepUtilizado.setText(Html.fromHtml("O último CEP utilizado foi " + "<b>" + ultimoCEPComMascara + "</b>" + ". Deseja mantê-lo?"));

        CheckBox checkBox = (CheckBox) dialog.findViewById(R.id.checkBoxNaoExibirMensagemNovamente);

        checkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {

                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    checkBoxSelecionada = true;
                }
            });



        Button SIMbutton = (Button) dialog.findViewById(R.id.SIMButton);
        SIMbutton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                ResultadosFragment.cepUsuario = ultimoCEP;
                if(checkBoxSelecionada){
                    SharedPreferences NaoExibirMaisPopupDeCepSharedPreferences = getSharedPreferences(
                    "NaoExibirMaisPopupDeCep", Context.MODE_PRIVATE);
                    NaoExibirMaisPopupDeCepSharedPreferences.edit().putString("modoSelecionado", "NÃO EXIBIR NOVAMENTE").apply();
                }
                dialog.dismiss();
            }
        });

        Button NAOButton = (Button) dialog.findViewById(R.id.NAOButton);
        NAOButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                dialogCEP();
                dialog.dismiss();
            }
        });

        dialog.show();

    }

    private AlertDialog alertDialogParaAvisoSobreLigarGPS() {
        return new AlertDialog.Builder(this)
                .setTitle("GPS!")
                .setMessage("Favor verificar se seu GPS encontra-se ligado, antes de utilizar essa opção")
                .setNegativeButton("OK", null)
                .show();
    }

    private void calcularLatitudeELongitueEExibirMensagemDeConfirmacaoDeCep() {

       // alertDialogParaAvisoSobreLigarGPS();

        final LocationListener locationListenerNetwork = sobrescreverMetodosObrigatoriosEmLocationListener();

        //Solicita permissão para uso do GPS do dispositivo móvel
        while ((ActivityCompat.checkSelfPermission(mn, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mn, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED)) {
            if (ActivityCompat.checkSelfPermission(mn, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mn, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    ActivityCompat.requestPermissions(mn, new String[]{Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.INTERNET}, 1);
                }
            }
        }

        //Pega os location providers
        Criteria criteria = new Criteria();
        criteria.setAccuracy(Criteria.ACCURACY_COARSE);
        List<String> lProviders = locationManager.getProviders(false);
        for (int i = 0; i < lProviders.size(); i++) {
            Log.d("LocationActivity", lProviders.get(i));
        }

        //Pega o melhor dentre os providers oferecidos
        String provider = locationManager.getBestProvider(criteria, false); // null

        //Toast.makeText(MainActivity.this, "provider = " + provider, Toast.LENGTH_LONG).show();
        System.out.println(provider);

        //Se esse provider for o gps, e não uso de conexão 3g, por exemplo
        //verifica se o GPS está ligado, se não estiver manda uma alert message para ligá-lo
        if (provider.equals("gps")) {
            final LocationManager manager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

            if (!manager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                buildAlertMessageNoGps();
            }
        }

        //Tenta recuperar as coordenadas via GPS
        locationManager.requestLocationUpdates(lProviders.get(0), 5000, 0, locationListenerNetwork);
        Location l = locationManager.getLastKnownLocation(lProviders.get(0));

        //Enquanto não conseguir recuperar, continuar tentando com outros providers
        while (l == null) {

            if (l == null) {
                locationManager.requestLocationUpdates(lProviders.get(1), 5000, 0, locationListenerNetwork);
                l = locationManager.getLastKnownLocation(lProviders.get(1));
            }

            if (l == null) {
                locationManager.requestLocationUpdates(lProviders.get(2), 5000, 0, locationListenerNetwork);
                l = locationManager.getLastKnownLocation(lProviders.get(2));
            }
        }

        //Último caso, embora desnecessário
        if (l == null) {
            Toast.makeText(MainActivity.this, "Erro ao acessar informações de GPS", Toast.LENGTH_LONG).show();
        } else {
            // A partir das informações recuperadas de latitude e longitude, usar o geocoder
            // para transformá-la em CEP
            Double latitude = l.getLatitude();
            Double longitude = l.getLongitude();
            Geocoder geocoder;
            List<Address> addresses;
            geocoder = new Geocoder(EstatisticasFragment.rootView.getContext(), Locale.getDefault());

            try {
                addresses = geocoder.getFromLocation(latitude, longitude, 1);
                cepUsuarioLocal = addresses.get(0).getPostalCode();
                if (cepUsuarioLocal.equals(null)){
                    wait(1000);
                    cepUsuarioLocal = addresses.get(0).getPostalCode();
                }
                if (cepUsuarioLocal.equals(null)){
                    cepRetornouNull = ERROR_CEP_NULL;
                    dialogCEP();
                }

                String addressLine = addresses.get(0).getAddressLine(0);
                System.out.println(addressLine);

                //Caso retorne apensar 5 digitos (geocoder as vezes faz isso), insira 000 no fim
                if (cepUsuarioLocal != null && cepUsuarioLocal.length() == 5)
                    cepUsuarioLocal = cepUsuarioLocal + "-000";
                System.out.println("cepUsuario: " + cepUsuarioLocal + "\nadressLine" + addressLine);
                //Trabalho para colocar hifen e ponto
                cepUsuarioLocalComHifen = cepUsuarioLocal;
                cepUsuarioLocal = cepUsuarioLocal.replace("-", "");
                System.out.println("cepUsuario: " + cepUsuarioLocal + "\nadressLine" + addressLine);
                //Toast.makeText(MainActivity.this, "cepUsuario: " + cepUsuarioLocal + "\nadressLine" + addressLine, Toast.LENGTH_LONG).show();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }


        }
    }

    private Dialog configurarDialog() {
        Dialog dialog = new Dialog(this);
        if (cepRetornouNull == false) {
            dialog.setContentView(R.layout.first_popup);
        } else if (cepRetornouNull == true){
            dialog.setContentView(R.layout.second_popup);
        }
        dialog.setTitle("Localização");

        /**TextView titleView = (TextView)dialog.findViewById(android.R.id.title);
         titleView.setGravity(Gravity.CENTER);**/

        // Make us non-modal, so that others can receive touch events.
        dialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL, WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL);

        // ...but notify us that it happened.
        dialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_WATCH_OUTSIDE_TOUCH, WindowManager.LayoutParams.FLAG_WATCH_OUTSIDE_TOUCH);
        return dialog;
    }

    private Dialog configurarDialogCEPJaExistente(final String ultimoCEP) {



        Dialog dialog =  new Dialog(this);
        dialog.setTitle("Localização");
        dialog.setContentView(R.layout.popup_cep_existente);
        // Make us non-modal, so that others can receive touch events.
        dialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL, WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL);

        // ...but notify us that it happened.
        dialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_WATCH_OUTSIDE_TOUCH, WindowManager.LayoutParams.FLAG_WATCH_OUTSIDE_TOUCH);

        return dialog;
    }

    private LocationListener sobrescreverMetodosObrigatoriosEmLocationListener() {
        return new LocationListener() {
            public void onLocationChanged(final Location location) {
            }

            @Override
            public void onStatusChanged(String s, int i, Bundle bundle) {
            }

            @Override
            public void onProviderEnabled(String s) {
            }

            @Override
            public void onProviderDisabled(String s) {
            }
        };
    }

    private void buildAlertMessageNoGps() {
        final AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage("Seu GPS parece estar desativado, deseja ativá-lo?")
                .setCancelable(false)
                .setPositiveButton("Sim", new DialogInterface.OnClickListener() {
                    public void onClick(@SuppressWarnings("unused") final DialogInterface dialog, @SuppressWarnings("unused") final int id) {
                        startActivity(new Intent(android.provider.Settings.ACTION_LOCATION_SOURCE_SETTINGS));
                    }
                })
                .setNegativeButton("Não", new DialogInterface.OnClickListener() {
                    public void onClick(final DialogInterface dialog, @SuppressWarnings("unused") final int id) {
                        dialog.cancel();
                    }
                });
        final AlertDialog alert = builder.create();
        alert.show();
    }

    // INÍCIO -- exibirMensagemConfigurarCep()

    private void exibirMensagemConfirmarCep() {

        String message = "O CEP encontrado para suas definições de latitude e longitude foi " +
                "<b>" + cepUsuarioLocalComHifen + "</b>" + ". Deseja mantê-lo?";

        final AlertDialog dialog = criarESetarTituloMensagemEBotoesDeAlertDialog(message);

        configurarBotoesPositivoENegativoDaAlertDialog(dialog);

        cepRetornouNull = RESET_ERROR_VARIABLE;
        dialog.show();

    }

    private AlertDialog criarESetarTituloMensagemEBotoesDeAlertDialog(String message) {
        return new AlertDialog.Builder(this)
                .setTitle("Confirmar CEP")
                .setMessage(Html.fromHtml(message))
                .setPositiveButton(android.R.string.ok, null) //Set to null. We override the onclick
                .setNegativeButton(android.R.string.cancel, null)
                .create();
    }

    private void configurarBotoesPositivoENegativoDaAlertDialog(AlertDialog dialog) {
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(final DialogInterface dialog) {

                ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {

                        ResultadosFragment.cepUsuario = cepUsuarioLocal;

                        SharedPreferences CEPsArmazenadosSharedPreferences = getSharedPreferences(
                                "CEPsArmazenados", Context.MODE_PRIVATE);
                        CEPsArmazenadosSharedPreferences.edit().putString("ultimoCEP", cepUsuarioLocal).apply();

                        SharedPreferences NaoExibirMaisPopupDeCepSharedPreferences = getSharedPreferences(
                                "NaoExibirMaisPopupDeCep", Context.MODE_PRIVATE);
                        NaoExibirMaisPopupDeCepSharedPreferences.edit().putString("modoSelecionado", "NOVO CEP INSERIDO").apply();

                            dialog.dismiss();


                        }


                });

                ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_NEGATIVE).setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {
                        dialog.dismiss();
                        dialogCEP();
                    }

                });
            }
        });
    }

    // FIM -- exibirMensagemConfigurarCep()

    // INÍCIO -- exibirMensagemInserirCep()

    private void exibirMensagemInserirCep() {

        //Abre o teclado
        final EditText input = instanciateAndSetKeyboardConfigurationsForInputEditText();

        final AlertDialog dialog = criarESetarTituloMensagemEBotoesDeAlertDialogInserirCep(input);

        configurarBotaoPositivoDaAlertDialog(dialog, input);
        cepRetornouNull = RESET_ERROR_VARIABLE;
        dialog.show();

    }

    private EditText instanciateAndSetKeyboardConfigurationsForInputEditText() {
        EditText input = new EditText(rf.resultadosFragmentContext);
        input.setRawInputType(Configuration.KEYBOARD_12KEY);
        input.setInputType(InputType.TYPE_CLASS_NUMBER);
        return input;
    }

    private AlertDialog criarESetarTituloMensagemEBotoesDeAlertDialogInserirCep(EditText input) {

        //Insere o CEP numa máscara
        MaskEditTextChangedListener maskCPF = new MaskEditTextChangedListener("##.###-###", input);
        input.addTextChangedListener(maskCPF);

        //Pega CEP inserido anteriormente, para exibir como forma rapida de uso
        SharedPreferences CEPsArmazenadosSharedPreferences = getSharedPreferences(
                "CEPsArmazenados", Context.MODE_PRIVATE);
            //System.out.println(CEPsArmazenadosSharedPreferences);
            final String ultimoCEP = CEPsArmazenadosSharedPreferences.getString("ultimoCEP", null);

            //Se existir CEP antigo, usá-lo como forma de acesso rápido
            if (ultimoCEP != null) {
            final String ultimoCEPComMascara = ultimoCEP.substring(0, 2) + "." + ultimoCEP.substring(2, 5) + "-" + ultimoCEP.substring(5, ultimoCEP.length());
            System.out.println(ultimoCEP);

            return new AlertDialog.Builder(rf.resultadosFragmentContext)
                    .setView(rootView)
                    .setTitle("Inserir CEP")
                    .setView(input)
                    .setNeutralButton("USAR " + ultimoCEPComMascara, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface d, int i) {
                            ResultadosFragment.cepUsuario = ultimoCEP;
                            SharedPreferences NaoExibirMaisPopupDeCepSharedPreferences = getSharedPreferences(
                                    "NaoExibirMaisPopupDeCep", Context.MODE_PRIVATE);
                            NaoExibirMaisPopupDeCepSharedPreferences.edit().putString("modoSelecionado", "NOVO CEP INSERIDO").apply();

                        }
                    })
                    .setPositiveButton(android.R.string.ok, null) //Set to null. We override the onclick
                    .create();
        } else {
            return new AlertDialog.Builder(rf.resultadosFragmentContext)
                    .setView(rootView)
                    .setTitle("Inserir CEP")
                    .setView(input)
                    .setPositiveButton(android.R.string.ok, null) //Set to null. We override the onclick
                    .create();
        }
    }

    private void configurarBotaoPositivoDaAlertDialog(AlertDialog dialog, final EditText input) {
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(final DialogInterface dialog) {

                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {

                        if ((input.getText().toString().replace(".", "").replace("-", "").length()) < 8){
                            Toast toast = Toast.makeText(rootView.getContext(), "CEP com menos de 8 dígitos. Tente novamente.", Toast.LENGTH_LONG);

                            toast.show();
                            exibirMensagemInserirCep();
                        }

                        if(QueryUtils.isCEPValido(input.getText().toString().replace(".", "").replace("-", ""))) {
                            System.out.println(input.getText().toString().replace(".", "").replace("-", ""));
                            ResultadosFragment.cepUsuario = input.getText().toString().replace(".", "").replace("-", "");

                            SharedPreferences CEPsArmazenadosSharedPreferences = getSharedPreferences(
                                    "CEPsArmazenados", Context.MODE_PRIVATE);
                            CEPsArmazenadosSharedPreferences.edit().putString("ultimoCEP", input.getText().toString().replace(".", "").replace("-", "")).apply();

                            SharedPreferences NaoExibirMaisPopupDeCepSharedPreferences = getSharedPreferences(
                                    "NaoExibirMaisPopupDeCep", Context.MODE_PRIVATE);
                            NaoExibirMaisPopupDeCepSharedPreferences.edit().putString("modoSelecionado", "NOVO CEP INSERIDO").apply();

                        } else {
                            Toast toast = Toast.makeText(rootView.getContext(), "CEP Inválido. Insira novamente.", Toast.LENGTH_LONG);
                            toast.show();
                            exibirMensagemInserirCep();
                        }

                        //Dismiss once everything is OK.
                        dialog.dismiss();
                        ResultadosFragment.rf.mAdapter.notifyDataSetChanged();
                        ResultadosFragment.esconderTeclado(MainActivity.this);

                    }

                });
            }
        });
    }

    // FIM -- exibirMensagemInserirCep()

    //Método voltado para a realização de uma pesquisa
    private void realizarPesquisa() {

        //Esconder o teclado após o início da pesquisa
        EstatisticasFragment.esconderTeclado(MainActivity.this);
        //Limpeza dos adapters que possam estar com sujeira de pesquisas prévias
        resetarResultadosFragment();
        resetarDeletadosFragment();

        NetworkInfo networkInfo = solicitarInformacoesDeConexao();
        // Início da conexão com as APIs do Mercado Livre, Correios, ViaCEP e Google Maps
        if (networkInfo != null && networkInfo.isConnected()) {
            //Limpeza da aba de estatísticas, que pode conter sujeira de pesquisas prévias
            esconderViewsDeEstatisticasFragment();
            //Exibe uma barra de progresso (loading), em todos os 3 fragments
            exibirProgressBarETextViewEmTodosOsFragments();
            //Início do request de dados
            inicializarLoader();
        } else {
            // Em caso de erro
            // Esconder a barra de progresso
            esconderProgressBarETextViewEmTodosOsFragments();
            // Exibir mensagem de conexão não existente
           exibirMensagemDeAusenciaDeConexaoEmTodosOsFragments();
        }
        //getLoaderManager().restartLoader(0, null, this);
    }

    private void exibirMensagemDeAusenciaDeConexaoEmTodosOsFragments() {
        TextView mEmptyStateTextViewEstatisticas = (TextView) findViewById(R.id.empty_view_estatisticas);
        mEmptyStateTextViewEstatisticas.setText(R.string.no_internet_connection);

        TextView mEmptyStateTextViewResultados = (TextView) findViewById(R.id.empty_view_resultados);
        mEmptyStateTextViewResultados.setText(R.string.no_internet_connection);

        TextView mEmptyStateTextViewDeletados = (TextView) findViewById(R.id.empty_view_deletados);
        mEmptyStateTextViewDeletados.setText(R.string.no_internet_connection);
    }

    // Verifica o status da conexão
    private NetworkInfo solicitarInformacoesDeConexao() {
        ConnectivityManager connMgr = (ConnectivityManager)
        MainActivity.this.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connMgr.getActiveNetworkInfo();
        return networkInfo;
    }

    private void inicializarLoader() {
        // Pega a referencia para o loaderManager
        LoaderManager loaderManager = getSupportLoaderManager();
        // Inicializa o loader, via loaderManager
        loaderManager.restartLoader(0, null, this);
    }

    private void resetarDeletadosFragment() {
        if (isDeletadorFragmentMAdapterNotNull()) {
            DeletadosFragment.df.mDeletadosAdapter.clear();
        }

        DeletadosFragment.df.recordsDeletadosGlobal.clear();
    }

    private void resetarResultadosFragment() {
        if (isResultadosFragmentMAdapterNotNull()) {
            ResultadosFragment.rf.mAdapter.clear();
        }

        ResultadosFragment.recordsGlobal.clear();
    }

    private void esconderViewsDeEstatisticasFragment() {
        esconderImageViewContendoImagemPrincipalEmEstatisticasFragment();
        esconderTextViewContendoMediaEmEstatisticasFragment();
        //esconderTextViewContendoTotalDeRegistrosDesvioPadraoECoeficienteDeVariacaoEmEstatisticasFragment();
    }

    /**private void esconderTextViewContendoTotalDeRegistrosDesvioPadraoECoeficienteDeVariacaoEmEstatisticasFragment() {
        TextView textViewTotalRegistros = (TextView) MainActivity.this.findViewById(R.id.textViewTotalRegistros);
        textViewTotalRegistros.setVisibility(View.GONE);
    }**/

    private void esconderTextViewContendoMediaEmEstatisticasFragment() {
        TextView textViewMedia = (TextView) MainActivity.this.findViewById(R.id.textViewMedia);
        textViewMedia.setVisibility(View.GONE);
    }

    private void esconderImageViewContendoImagemPrincipalEmEstatisticasFragment() {
        ImageView imageView = (ImageView) MainActivity.this.findViewById(R.id.imageViewImagemPrincipal);
        imageView.setVisibility(View.GONE);
    }

    private boolean isDeletadorFragmentMAdapterNotNull() {
        return DeletadosFragment.df.mDeletadosAdapter != null;
    }

    private boolean isResultadosFragmentMAdapterNotNull() {
        return ResultadosFragment.rf.mAdapter != null;
    }

    // FINAL - realizarPesquisa()

    //INÍCIO - exibirProgressBarETextViewEmTodosOsFragments()

    private void exibirProgressBarETextViewEmTodosOsFragments() {
        showProgressBarETextViewEmEstatisticasFragment();
        showProgressBarETextViewEmResultadosFragment();
        showProgressBarETextViewEmDeletadosFragment();
    }

    private void showProgressBarETextViewEmEstatisticasFragment() {
        ProgressBar loadingIndicatorEstatisticas = (ProgressBar) EstatisticasFragment.rootView.findViewById(R.id.loading_indicator);
        loadingIndicatorEstatisticas.setVisibility(View.VISIBLE);
        TextView textViewloadingIndicatorEstatisticas = (TextView) EstatisticasFragment.rootView.findViewById(R.id.textView_loading_indicator);
        textViewloadingIndicatorEstatisticas.setVisibility(View.VISIBLE);
    }

    private void showProgressBarETextViewEmResultadosFragment() {
        ProgressBar loadingIndicatorResultados = (ProgressBar) ResultadosFragment.rootView.findViewById(R.id.loading_indicator_resultados);
        loadingIndicatorResultados.setVisibility(View.VISIBLE);
        TextView textViewloadingIndicatorResultados = (TextView) ResultadosFragment.rootView.findViewById(R.id.textView_loading_indicator_resultados);
        textViewloadingIndicatorResultados.setVisibility(View.VISIBLE);
    }

    private void showProgressBarETextViewEmDeletadosFragment() {
        ProgressBar loadingIndicatorDeletados = (ProgressBar) findViewById(R.id.loading_indicator_deletados);
        loadingIndicatorDeletados.setVisibility(View.VISIBLE);
        TextView textViewloadingIndicatorDeletados = (TextView) findViewById(R.id.textView_loading_indicator_deletados);
        textViewloadingIndicatorDeletados.setVisibility(View.VISIBLE);
    }

    private void esconderProgressBarETextViewEmTodosOsFragments() {
        hideProgressBarETextViewEmEstatisticasFragment();
        hideProgressBarETextViewEmResultadosFragment();
        hideProgressBarETextViewEmDeletadosFragment();
    }

    private void hideProgressBarETextViewEmEstatisticasFragment() {
        ProgressBar loadingIndicatorEstatisticas = (ProgressBar) EstatisticasFragment.rootView.findViewById(R.id.loading_indicator);
        loadingIndicatorEstatisticas.setVisibility(View.GONE);
        TextView textViewloadingIndicatorEstatisticas = (TextView) EstatisticasFragment.rootView.findViewById(R.id.textView_loading_indicator);
        textViewloadingIndicatorEstatisticas.setVisibility(View.GONE);
    }

    private void hideProgressBarETextViewEmResultadosFragment() {
        ProgressBar loadingIndicatorResultados = (ProgressBar) ResultadosFragment.rootView.findViewById(R.id.loading_indicator_resultados);
        loadingIndicatorResultados.setVisibility(View.GONE);
        TextView textViewloadingIndicatorResultados = (TextView) ResultadosFragment.rootView.findViewById(R.id.textView_loading_indicator_resultados);
        textViewloadingIndicatorResultados.setVisibility(View.GONE);
    }

    private void hideProgressBarETextViewEmDeletadosFragment() {
        ProgressBar loadingIndicatorDeletados = (ProgressBar) DeletadosFragment.rootView.findViewById(R.id.loading_indicator_deletados);
        loadingIndicatorDeletados.setVisibility(View.GONE);
        TextView textViewloadingIndicatorDeletados = (TextView) DeletadosFragment.rootView.findViewById(R.id.textView_loading_indicator_deletados);
        textViewloadingIndicatorDeletados.setVisibility(View.GONE);
    }

    //FINAL - exibirProgressBarETextViewEmTodosOsFragments()

    public boolean filtrarResultadosJSON(String tituloDoAnuncio, String condition) {
        //Extrai o nome do Álbum do título do anúncio
        String[] splitedAlbum = searchAlbum.split("\\s+");
        //System.out.println("splitedAlbum = " + splitedAlbum);
        //Extrai o nome da Banda do título do anúncio
        String[] splitedBand = searchBand.split("\\s+");
        //System.out.println("splitedBand = " + splitedBand);

        String tituloSemAlbumEBanda;
        String albumNormalizadoEmString;
        String bandaNormalizadaEmString;

        //Remoção de acentos e caracteres especiais
        String tituloNormalizado = Normalizer.normalize(tituloDoAnuncio, Normalizer.Form.NFD);
        tituloDoAnuncio = tituloNormalizado.replaceAll("[^\\p{ASCII}]", "");

        //Dois for, para dar como válido somente os anúncios que tem, no título, tanto a banda quanto o álbum
        for (int i = 0; i < splitedAlbum.length; i++) {
            albumNormalizadoEmString = normalizarString(splitedAlbum[i]);
            splitedAlbum[i] = removerAcentosDeString(albumNormalizadoEmString);
            //Verifica se algum título de anúncio não contém o álbum, se não tiver, descarta o resultado
            if (!tituloDoAnuncio.toLowerCase().contains(splitedAlbum[i].toLowerCase())) {
                System.out.println(">>>>>> Vou retornar FALSE pois o título do anúncio não contém o álbum solicitado");
                return false;
            }
        }
        for (int i = 0; i < splitedBand.length; i++) {
            bandaNormalizadaEmString = normalizarString(splitedBand[i]);
            splitedBand[i] = removerAcentosDeString(bandaNormalizadaEmString);
            //Verifica se algum título de anúncio não contém a banda, se não tive, descarta o resultado
            if (!tituloDoAnuncio.toLowerCase().contains(splitedBand[i].toLowerCase())) {
                System.out.println(">>>>>> Vou retornar FALSE pois o título do anúncio não contém a banda solicitada");
                return false;
            }
        }

        //Popula uma variável com o resto do título do anúncio, mas sem o álbum e banda
        tituloSemAlbumEBanda = tituloDoAnuncio.toLowerCase().replace(searchAlbum.toLowerCase(), "");
        tituloSemAlbumEBanda = tituloSemAlbumEBanda.replace(searchBand.toLowerCase(), "");

        //System.out.println(">>> Título do Anúncio Sem Álbum E Banda = " + tituloSemAlbumEBanda);

        //Dicionário de dados, para, caso o título contenha algum desses termos, o resultado seja descartado
        if ((tituloSemAlbumEBanda.toLowerCase().contains("cd ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains("cd-")) ||
        (tituloSemAlbumEBanda.toLowerCase().contains(" cd-")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" cd ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" cd"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("cd's ")) || (tituloSemAlbumEBanda.toLowerCase().contains(" cd's ")) || (tituloSemAlbumEBanda.toLowerCase().contains(" cd's"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("dvd ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" dvd ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" dvd"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("capa ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" capa ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" capa"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("k7 ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" k7 ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" k7"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("fita cassete ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" fita cassete ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" fita cassete"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("decoração ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" decoração ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" decoração"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("artesanato ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" artesanato ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" artesanato"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("enfeites ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" enfeites ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" enfeites"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        if ((tituloSemAlbumEBanda.toLowerCase().contains("decorações ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" decorações ")) ||
                (tituloSemAlbumEBanda.toLowerCase().contains(" decorações"))) {
            System.out.println(">>>>>> Vou retornar FALSE pois não passei pelo filtro");
            return false;
        }

        //Filtragem de "novo" e "usado"
        if (opcaoNovoSelecionadaNoFiltroAntesDaPesquisa) {
            if (condition.contains("used")) {
                System.out.println(">>>>>> Sou USADO e vou retornar FALSO, pois a pesquisa solicita NOVO");
                return false;
            }
        } else if (opcaoUsadoSelecionadaNoFiltroAntesDaPesquisa) {
            if (condition.contains("new")) {
                System.out.println(">>>>>> Sou NOVO e vou retornar FALSO, pois a pesquisa solicita USADO");
                return false;
            }
        }

        //Filtragem de "nacional" e "importado"
        if (opcaoNacionalSelecionadaNoFiltroAntesDaPesquisa) {
            if (tituloSemAlbumEBanda.toLowerCase().contains("imp.") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("imp ") ||
                    tituloSemAlbumEBanda.toLowerCase().contains(" imp ") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("import") ||
                    tituloSemAlbumEBanda.toLowerCase().contains(" imp") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(imp)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("americano") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(americano)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("usa") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("usa") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(usa)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("u.s.a") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(u.s.a)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("japan") &&
                    tituloSemAlbumEBanda.toLowerCase().contains("(japan)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("japanese") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(japanese)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(japones)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("japones") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(japão)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("japão") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("japao") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(japao)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("uk") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(uk)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("alemao") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(alemao)") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("jamaica") ||
                    tituloSemAlbumEBanda.toLowerCase().contains("(jamaica)")) {
                System.out.println(">>>>>> Sou IMPORTADO e vou retornar FALSO, pois a pesquisa solicita NACIONAL");
                return false;
            }
        } else if (opcaoImportadoSelecionadaNoFiltroAntesDaPesquisa) {
            System.out.println(tituloSemAlbumEBanda.toLowerCase());
            if (!tituloSemAlbumEBanda.contains("imp.") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("imp ") &&
                !tituloSemAlbumEBanda.toLowerCase().contains(" imp ") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("import") &&
                !tituloSemAlbumEBanda.toLowerCase().contains(" imp") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(imp)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("americano") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(americano)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("usa") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("usa") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(usa)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("u.s.a") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(u.s.a)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("japan") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(japan)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("japanese") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(japanese)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(japones)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("japones") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(japão)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("japão") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("japao") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(japao)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("uk") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(uk)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("alemao") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(alemao)") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("jamaica") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("(jamaica)"))
                {

                    System.out.println(">>>>>> Sou NACIONAL e vou retornar FALSO, pois a pesquisa solicita IMPORTADO");
                    return false;

                }

            if (tituloSemAlbumEBanda.toLowerCase().contains("impecavel")){
                return false;
            }
        }

        //Filtro de compacto (7'') e longplay (12'')
        if (opcaoCompactoSelecionadaNoFiltroAntesDaPesquisa) {
            if (!tituloSemAlbumEBanda.toLowerCase().contains("compacto") &&
                !tituloSemAlbumEBanda.toLowerCase().contains("single")) {
                System.out.println(">>>>>> Sou LONGPLAY e vou retornar FALSO, pois a pesquisa solicita COMPACTO");
                return false;
            }
        } else if (opcaoLongPlaySelecionadaNoFiltroAntesDaPesquisa) {
            if (tituloSemAlbumEBanda.toLowerCase().contains("compacto") ||
                tituloSemAlbumEBanda.toLowerCase().contains("single")) {
                System.out.println(">>>>>> Sou COMPACTO e vou retornar FALSO, pois a pesquisa solicita LONGPLAY");
                return false;
            }
        }
        System.out.println(">>>>>> Passei por TODAS as validações e vou retornar TRUE");
        return true;
    }

    private String removerAcentosDeString(String stringComAcentosASeremREmovidos) {
        return stringComAcentosASeremREmovidos.replaceAll("[^\\p{ASCII}]", "");
    }

    private String normalizarString(String stringASerNormalizada) {
        return Normalizer.normalize(stringASerNormalizada, Normalizer.Form.NFD);
    }

    protected void onSaveInstanceState(Bundle outState) {
        // salva a Tab selecionada
        outState.putString("tab", mTabHost.getCurrentTabTag());
        super.onSaveInstanceState(outState);
    }

    private void intialiseViewPager() {


        List<Fragment> fragments = new Vector<Fragment>();
        fragments.add(Fragment.instantiate(this, EstatisticasFragment.class.getName()));
        fragments.add(Fragment.instantiate(this, ResultadosFragment.class.getName()));
        fragments.add(Fragment.instantiate(this, DeletadosFragment.class.getName()));

        SimpleFragmentPagerAdapter adapter = new SimpleFragmentPagerAdapter(
                super.getSupportFragmentManager(), fragments);

        this.viewPager = (ViewPager) super.findViewById(R.id.viewpager);
        this.viewPager.setAdapter(adapter);
        this.viewPager.setOffscreenPageLimit(3); //importante para não fazer refresh ao trocar de fragment
        this.viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {

            @Override
            public void onPageSelected(int pos) {
                mTabHost.setCurrentTab(pos);
            }

            @Override
            public void onPageScrolled(int arg0, float arg1, int arg2) {
                // TODO Auto-generated method stub

            }

            @Override
            public void onPageScrollStateChanged(int pos) {

            }
        });

    }

}


