package com.example.android.vinylfinder;

import android.location.Address;
import android.location.Geocoder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;
import android.util.Xml;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;


public final class QueryUtils {

    private static final String LOG_TAG = QueryUtils.class.getSimpleName();

    //Endpoint do ML para informações sobre vendedores
    private static final String SELLER_REQUEST_URL =
            "https://api.mercadolibre.com/users";

    //Endpoint do ML para informações sobre anuncios
    private static final String DATA_ANUNCIO_REQUEST_URL =
            "https://api.mercadolibre.com/items";

    //Endpoint dos Correios
    private static final String CORREIOS_REQUEST_SEDEX_URL = "http://ws.correios.com.br/calculador/CalcPrecoPrazo.aspx?nCdEmpresa=&sDsSenha=&nVlPeso=1&nCdFormato=1&nVlComprimento=45&nVlAltura=45&nVlLargura=11&sCdMaoPropria=n&nVlValorDeclarado=0&sCdAvisoRecebimento=n&nCdServico=04014&nVlDiametro=0&StrRetorno=xml&nIndicaCalculo=3";
    private static final String CORREIOS_REQUEST_PAC_URL = "http://ws.correios.com.br/calculador/CalcPrecoPrazo.aspx?nCdEmpresa=&sDsSenha=&nVlPeso=1&nCdFormato=1&nVlComprimento=45&nVlAltura=45&nVlLargura=11&sCdMaoPropria=n&nVlValorDeclarado=0&sCdAvisoRecebimento=n&nCdServico=04510&nVlDiametro=0&StrRetorno=xml&nIndicaCalculo=3";

    //Endpoint do ViaCep
    private static final String VIACEP_REQUEST_URL = "https://viacep.com.br/ws/";

    private static String sellerReputation;
    public static String postalCode;
    private static Date dataAnuncio;
    private static final String ns = null;
    private static Double precoSedex = null;
    private static Double precoPac = null;
    private static Double latitude;
    private static Double longitude;
    private static Thread threadParaPegarInformacoesDoMercadoEnviosMercadoEnvios;
    private static Thread threadParaPegarInformacoesDoCustom;
    private static Thread threadParaPegarInformacoesDosCorreios;
    private static String anuncioId;
    private static String sellerId;
    private static String cityState;
    private static boolean freeShiping;
    private static String title;
    private static Double price;
    private static String condition;
    private static String thumbnail;
    private static String permalink;
    private static String shippingMode;
    private static String CodigoDeErro;
    public static String tagErro;
    private static Double mercadoEnviosExpresso;
    private static Double mercadoEnviosNormal;
    private static Double mercadoLivreCustomPrice;
    private static String urlPictureBig;
    private static String urlPictureBigSize;
    private static int contador;
    private static int acumuladorResultados;

    private QueryUtils() {
    }

    /**
     * Realiza pesquisa na base de dados do Mercado Livre e retorna uma lista de objetos Record
     */
    public static List<Record> fetchRecordData(String requestUrl) {
        // Extrai os campos importantes do JSON retornado pelo Mercado Livre
        return extractFeatureFromJson(instanciarUrlEFazerRequestHttp(requestUrl));
    }

    public static int fetchTotalOfRecords(String requestUrl) {
        return extractFeatureFromTotalOfRecord(instanciarUrlEFazerRequestHttp(requestUrl));
    }

    private static int extractFeatureFromTotalOfRecord(String recordJSON) {
        int totalDeResultadosDaBusca = 0;
        if (TextUtils.isEmpty(recordJSON)) {
            return 0;
        }

        List<Record> records = new ArrayList<>();

        try {

            JSONObject baseJsonResponse = new JSONObject(recordJSON);

            JSONObject pagingObject = baseJsonResponse.getJSONObject("paging");

            totalDeResultadosDaBusca = pagingObject.getInt("total");

        } catch (JSONException e) {
            e.printStackTrace();
        }
        return totalDeResultadosDaBusca;
    }

    public static String fetchSellerData(String requestUrl) {

        return extractFeatureFromSellerJson(instanciarUrlEFazerRequestHttp(requestUrl));

    }

    public static Date fetchAnuncioData(String requestUrl) {

        return extractFeatureFromDataJson(instanciarUrlEFazerRequestHttp(requestUrl));

    }

    public static String[] fetchMercadoEnvios(String requestUrl) {

        return extractFeatureFromMercadoEnviosJson(instanciarUrlEFazerRequestHttp(requestUrl));

    }

    public static String fetchCustomPrice(String requestUrl) {

        return extractFeatureFromCustomPriceJson(instanciarUrlEFazerRequestHttp(requestUrl));

    }

    public static String fetchAnuncioPicture(String requestUrl) {

        return extractFeatureFromPictureJson(instanciarUrlEFazerRequestHttp(requestUrl));

    }

    public static String instanciarUrlEFazerRequestHttp(String requestUrl) {
        // Objeto URL
        URL url = createUrl(requestUrl);

        // Realiza o request HTTP e recupera a resposta JSON
        String jsonResponse = null;
        try {
            jsonResponse = makeHttpRequest(url);
        } catch (IOException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e(LOG_TAG, "Problema no request HTTP", e);
            }
        }
        //Retorna a resposta JSON em String
        return jsonResponse;
    }

    /**
     * Returns new URL object from the given string URL.
     */
    private static URL createUrl(String stringUrl) {
        URL url = null;
        try {
            url = new URL(stringUrl);
        } catch (MalformedURLException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e(LOG_TAG, "Problema fazendo build da URL", e);
            }
        }
        return url;
    }

    private static String makeHttpRequest(URL url) throws IOException {
        String jsonResponse = "";

        if (url == null) {
            return jsonResponse;
        }

        HttpURLConnection urlConnection = null;
        InputStream inputStream = null;
        try {
            urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setReadTimeout(10000 /* milliseconds */);
            urlConnection.setConnectTimeout(5000 /* milliseconds */);
            urlConnection.setRequestMethod("GET");
            urlConnection.connect();

            //Se 200 - OK, ler a resposta JSON e colocar em InputStream
            if (urlConnection.getResponseCode() == 200) {
                inputStream = urlConnection.getInputStream();
                jsonResponse = readFromStream(inputStream);
            } else {
                Log.e(LOG_TAG, "Erro: " + urlConnection.getResponseCode());
            }
        } catch (IOException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e(LOG_TAG, "Problema recuperando os resultados JSON", e);
            }
        } finally {
            if (urlConnection != null) {
                urlConnection.disconnect();
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
        return jsonResponse;
    }

    /**
     * Converte InputStream em String
     */
    private static String readFromStream(InputStream inputStream) throws IOException {
        StringBuilder output = new StringBuilder();
        if (inputStream != null) {
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, Charset.forName("UTF-8"));
            BufferedReader reader = new BufferedReader(inputStreamReader);
            String line = reader.readLine();
            while (line != null) {
                output.append(line);
                line = reader.readLine();
            }
        }
        return output.toString();
    }

    /**
     * Retorna lista de records criadas a partir
     * do JSON retornado pelo Mercado Livre
     */
    private static List<Record> extractFeatureFromJson(String recordJSON) {
        acumuladorResultados = 0;
        contador = 0;

        if (TextUtils.isEmpty(recordJSON)) {
            return null;
        }

        // Lista vazia de Records
        List<Record> records = new ArrayList<>();

        // Try para realizar parse da string JSON.
        try {

            // Cria um JSONObject baseado na String retornada pelo Mercado Livre
            JSONObject baseJsonResponse = new JSONObject(recordJSON);

            // Extrai o JSONArray "results"
            JSONArray recordArray = baseJsonResponse.getJSONArray("results");

            //Filtro rápido para saber o total de resultados, e poder criar a barra de progresso
            acumuladorResultados = filtrarTodosOsResultadosApenasParaSetarOMaximoDeResultadosDuranteOCarregamentoDosRecords(recordArray);
            setarOMaximoDeResultadosDuranteOCarregamentoDosRecords();

            // Cria um objeto Record para cada anuncio retornado noo JSONArray "results"
            for (int i = 0; i < recordArray.length(); i++) {

                //Método onde o parse de informações básicas do anúncio são recuperadas
                solicitarInformacoesBasicasDoRecord(recordArray, i);

                if (resultadoPassouPeloFiltro()) {
                    contador++;

                    atualizarProgressoDoContadorDaProgressBar();

                    //Passagem das informações básicas recuperadas anteriormente, para pegar informações mais detalhadas
                    criarThreadsDiversasParaPegarInformacoesDoAnuncio(anuncioId, sellerId, shippingMode);

                    //Caso a tag free_shipping retorne true, setar os preços de Sedex e Pac para zero
                    if (freeShiping) {
                        precoSedex = 0.0;
                        precoPac = 0.0;
                    }

                    records.add(new Record(title, price, thumbnail, permalink, cityState, latitude, longitude, freeShiping, sellerReputation, dataAnuncio, precoSedex, precoPac, shippingMode, mercadoEnviosNormal, mercadoEnviosExpresso, mercadoLivreCustomPrice, urlPictureBig, urlPictureBigSize));
                }
            }
        //Catch para qualquer erro relacionado à operação
        } catch (JSONException e) {
            Log.e("QueryUtils", "Problema no parse de Records", e);
        }

        zerarVariaveisDeContador();
        // Retorna a lista de Records
        return records;
    }

    private static void zerarVariaveisDeContador() {
        contador = 0;
        acumuladorResultados = 0;

        EstatisticasFragment.ef.loadingIndicator.setMax(0);
        ResultadosFragment.rf.loadingIndicatorResultados.setMax(0);
        DeletadosFragment.df.loadingIndicatorDeletados.setMax(0);

        EstatisticasFragment.ef.loadingIndicator.setProgress(0);
        ResultadosFragment.rf.loadingIndicatorResultados.setProgress(0);
        DeletadosFragment.df.loadingIndicatorDeletados.setProgress(0);
    }

    private static boolean resultadoPassouPeloFiltro() {
        return MainActivity.mn.filtrarResultadosJSON(title, condition);
    }

    private static void atualizarProgressoDoContadorDaProgressBar() {
        MainActivity.mn.runOnUiThread(new Runnable() {
            @Override
            public void run() {

                EstatisticasFragment.ef.loadingIndicator.setProgress(contador);
                EstatisticasFragment.ef.textViewLoadingIndicator.setText(contador + " / " + acumuladorResultados);

                ResultadosFragment.rf.loadingIndicatorResultados.setProgress(contador);
                ResultadosFragment.rf.textViewLoadingIndicatorResultados.setText(contador + " / " + acumuladorResultados);

                DeletadosFragment.df.loadingIndicatorDeletados.setProgress(contador);
                DeletadosFragment.df.textViewLoadingIndicatorDeletados.setText(contador + " / " + acumuladorResultados);

            }
        });
    }

    private static void solicitarInformacoesBasicasDoRecord(JSONArray recordArray, int i) throws JSONException {
        //Referência para um registro de anúncio vindo do parse do JSON
        JSONObject currentResult = recordArray.getJSONObject(i);

        //Pega diversas informações do anúncio e armazena em variáveis
        anuncioId = currentResult.getString("id");
        title = currentResult.getString("title");
        price = currentResult.getDouble("price");
        condition = currentResult.getString("condition");
        thumbnail = currentResult.getString("thumbnail");
        permalink = currentResult.getString("permalink");
        //System.out.println("---------------------------------------------");
        //System.out.println(">>> Título do Anúncio = " + title);
        JSONObject shiping = currentResult.getJSONObject("shipping");
        freeShiping = shiping.getBoolean("free_shipping");
        shippingMode = shiping.getString("mode");
        //System.out.println(">>> Modo de Envio = " + shippingMode);

        //Pega diversas informações do vendedor e armazena em variáveis
        JSONObject seller = currentResult.getJSONObject("seller");
        sellerId = seller.getString("id");
        JSONObject sellerAddress = currentResult.getJSONObject("seller_address");
        JSONObject stateObject = sellerAddress.getJSONObject("state");
        final String state = stateObject.getString("id");
        JSONObject cityObject = sellerAddress.getJSONObject("city");
        final String city = cityObject.getString("name");
        final String statePuro = state.replace("BR-", "");
        cityState = city.concat(" - " + statePuro);

    }

    private static void criarThreadsDiversasParaPegarInformacoesDoAnuncio(String anuncioId, String sellerId, String shippingMode) {
        Runnable runnableParaPegarDataDoAnuncio = runnableParaPegarDataDoAnuncio(anuncioId);
        Runnable runnableParaPegarInformacoesDoMercadoEnvios = runnableParaPegarInformacoesDoMercadoEnvios(anuncioId);
        Runnable runnableParaPegarInformacoesDoCustom = runnableParaPegarInformacoesDoCustom(anuncioId);
        Runnable runnableParaPegarInformacoesDosCorreios = runnableParaPegarInformacoesDosCorreios();
        Runnable runnableParaPegarAReputacaoDoVendedor = runnableParaPegarReputacaoDoVendedor(sellerId);

        Thread threadParaPegarDataDoAnuncio = new Thread(runnableParaPegarDataDoAnuncio);
        Thread threadParaPegarAReputacaoDoVendedor = new Thread(runnableParaPegarAReputacaoDoVendedor);

        threadParaPegarDataDoAnuncio.start();
        threadParaPegarAReputacaoDoVendedor.start();

        //Se a variavel shippingMode, populada com as informações básicas, for de Me1 ou Me2, buscar informações de MercadoEnvios
        if (shippingMode.equals("me1") || shippingMode.equals("me2")) {
            threadParaPegarInformacoesDoMercadoEnviosMercadoEnvios = new Thread(runnableParaPegarInformacoesDoMercadoEnvios);
            threadParaPegarInformacoesDoMercadoEnviosMercadoEnvios.start();
        }
        //Se a variavel shippingMode, populada com as informações básicas, for de Custom, buscar info de Preço Tabelado
        if (shippingMode.equals("custom")) {
            threadParaPegarInformacoesDoCustom = new Thread(runnableParaPegarInformacoesDoCustom);
            threadParaPegarInformacoesDoCustom.start();
        }
        //Se a variavel shippingMode, populada com as informações básicas, for de not_specified, buscar info dos Correios
        if (shippingMode.equals("not_specified")) {
            threadParaPegarInformacoesDosCorreios = new Thread(runnableParaPegarInformacoesDosCorreios);
            threadParaPegarInformacoesDosCorreios.start();
        }

        //Faz o join da threads, dependendo da combinação
        try {
            if (shippingMode.equals("me1") || shippingMode.equals("me2")) {
                threadParaPegarDataDoAnuncio.join();
                threadParaPegarAReputacaoDoVendedor.join();
                threadParaPegarInformacoesDoMercadoEnviosMercadoEnvios.join();
                System.out.println(">>>>>> Preço Mercado Envios Expresso = " + mercadoEnviosExpresso);
                System.out.println(">>>>>> Preço Mercado Envios Normal = " + mercadoEnviosNormal);
            }
            if (shippingMode.equals("custom")) {
                threadParaPegarDataDoAnuncio.join();
                threadParaPegarAReputacaoDoVendedor.join();
                threadParaPegarInformacoesDoCustom.join();
                System.out.println(">>>>>> Preço Custom = " + mercadoLivreCustomPrice);
            }
            if (shippingMode.equals("not_specified")) {
                threadParaPegarDataDoAnuncio.join();
                threadParaPegarAReputacaoDoVendedor.join();
                threadParaPegarInformacoesDosCorreios.join();
            }

            System.out.println("---------------------------------------------");

            //threadPictureAnuncio.join();
        } catch (InterruptedException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                e.printStackTrace();
            }
        }
    }

    private static Runnable runnableParaPegarReputacaoDoVendedor(final String sellerId) {
        Runnable threadSellerReputation = new Runnable() {
            public void run() {
                Uri baseUri = Uri.parse(SELLER_REQUEST_URL);
                Uri.Builder uriBuilder = baseUri.buildUpon();
                uriBuilder.appendPath(sellerId);
                System.out.println(">>>>>> URL do usuário vendedor = " + uriBuilder.toString());
                sellerReputation = fetchSellerData(uriBuilder.toString());
            }
        };
        return threadSellerReputation;
    }

    private static Runnable runnableParaPegarInformacoesDosCorreios() {
        Runnable threadFrete = new Runnable() {
            public void run() {
                URL urlSedex;
                URL urlPac;
                try {
                    Uri baseUriSedex = Uri.parse(CORREIOS_REQUEST_SEDEX_URL);
                    Uri.Builder uriBuilderSedex = baseUriSedex.buildUpon();
                    uriBuilderSedex.appendQueryParameter("sCepOrigem", postalCode);
                    uriBuilderSedex.appendQueryParameter("sCepDestino", ResultadosFragment.cepUsuario);
                    System.out.println(">>>>>> URL de cálculo do frete dos correios = " + uriBuilderSedex.toString());
                    urlSedex = new URL(uriBuilderSedex.toString());
                    URLConnection urlConnectionSedex;
                    urlConnectionSedex = urlSedex.openConnection();
                    InputStream inSedex = new BufferedInputStream(urlConnectionSedex.getInputStream());
                    parse(inSedex);

                    Uri baseUriPac = Uri.parse(CORREIOS_REQUEST_PAC_URL);
                    Uri.Builder uriBuilderPac = baseUriPac.buildUpon();
                    uriBuilderPac.appendQueryParameter("sCepOrigem", postalCode);
                    uriBuilderPac.appendQueryParameter("sCepDestino", ResultadosFragment.cepUsuario);
                    System.out.println(">>>>>> URL de cálculo do frete dos correios = " + uriBuilderPac.toString());
                    urlPac = new URL(uriBuilderPac.toString());
                    URLConnection urlConnectionPac;
                    urlConnectionPac = urlPac.openConnection();
                    InputStream inPac = new BufferedInputStream(urlConnectionPac.getInputStream());
                    parse(inPac);

                    //Paliativo
                    if (precoSedex == 0 && precoPac == 0) {
                        System.out.println(">>>>>>>>>>>> Entrei no IF de precoSedex e precoPac == 0");
                        precoPac = 1000.0;
                        precoSedex = 1000.0;
                    }
                } catch (MalformedURLException e) {
                    if (e != null) {
                        e.printStackTrace();
                    } else {
                        e.printStackTrace();
                    }
                } catch (IOException e) {
                    if (e != null) {
                        e.printStackTrace();
                    } else {
                        e.printStackTrace();
                    }
                } catch (XmlPullParserException e) {
                    if (e != null) {
                        e.printStackTrace();
                    } else {
                        e.printStackTrace();
                    }
                }

            }
        };
        return threadFrete;
    }

    private static Runnable runnableParaPegarInformacoesDoCustom(final String anuncioId) {
        Runnable threadMercadoCustomRunnable = new Runnable() {
            public void run() {
                //Constrói a URL para pegar as informações de mercado Envios
                Uri baseUriMercadoEnvios = Uri.parse(DATA_ANUNCIO_REQUEST_URL);
                Uri.Builder uriBuilderMercadoEnvios = baseUriMercadoEnvios.buildUpon();
                uriBuilderMercadoEnvios.appendPath(anuncioId);
                uriBuilderMercadoEnvios.appendPath("shipping_options");
                uriBuilderMercadoEnvios.appendQueryParameter("zip_code", ResultadosFragment.cepUsuario);
                uriBuilderMercadoEnvios.appendQueryParameter("quantity", "1");
                System.out.println(">>>>>> URL Mercado Envios Custom = " + uriBuilderMercadoEnvios.toString());

                String mercadoLivreEnviosCustom = fetchCustomPrice(uriBuilderMercadoEnvios.toString());
                if (mercadoLivreEnviosCustom != null)
                    mercadoLivreCustomPrice = Double.parseDouble(mercadoLivreEnviosCustom.replace(",", "."));
            }
        };
        return threadMercadoCustomRunnable;
    }


    private static Runnable runnableParaPegarInformacoesDoMercadoEnvios(final String anuncioId) {
        Runnable threadMercadoEnviosRunnable = new Runnable() {
            public void run() {
                //Constrói a URL para pegar as informações de mercado Envios
                Uri baseUriMercadoEnvios = Uri.parse(DATA_ANUNCIO_REQUEST_URL);
                Uri.Builder uriBuilderMercadoEnvios = baseUriMercadoEnvios.buildUpon();
                uriBuilderMercadoEnvios.appendPath(anuncioId);
                uriBuilderMercadoEnvios.appendPath("shipping_options");
                uriBuilderMercadoEnvios.appendQueryParameter("zip_code", ResultadosFragment.cepUsuario);
                uriBuilderMercadoEnvios.appendQueryParameter("quantity", "1");
                System.out.println(">>>>>> URL Mercado Envios Me1 ou Me2 = " + uriBuilderMercadoEnvios.toString());
                String[] mercadoLivreEnvios = fetchMercadoEnvios(uriBuilderMercadoEnvios.toString());

                if (mercadoLivreEnvios[0] != null) {
                    mercadoEnviosExpresso = Double.parseDouble(mercadoLivreEnvios[0].replace(",", "."));
                }
                if (mercadoLivreEnvios[1] != null) {
                    mercadoEnviosNormal = Double.parseDouble(mercadoLivreEnvios[1].replace(",", "."));
                }

                //Validação de Frete Grátis
                if (mercadoLivreEnvios[0] != null && mercadoLivreEnvios[1] != null && mercadoLivreEnvios[1].equals(mercadoLivreEnvios[0])) {
                    mercadoEnviosNormal = 0.0;
                    mercadoLivreEnvios[1] = "0.0";
                }

            }
        };
        return threadMercadoEnviosRunnable;
    }


    private static Runnable runnableParaPegarDataDoAnuncio(final String anuncioId) {
        Runnable threadData = new Runnable() {
            public void run() {
                //Constrói a URL para pegar as informações de Data do Anuncio e Coordenadas do Vendedor
                Uri baseUriDataAnuncio = Uri.parse(DATA_ANUNCIO_REQUEST_URL);
                Uri.Builder uriBuilderDataAnuncio = baseUriDataAnuncio.buildUpon();
                uriBuilderDataAnuncio.appendPath(anuncioId);
                System.out.println(">>>>>> URL do Anúncio = " + uriBuilderDataAnuncio.toString());
                dataAnuncio = fetchAnuncioData(uriBuilderDataAnuncio.toString());
            }
        };
        return threadData;
    }

    private static void utilizarGeocoderParaDecifrarCepDoVendedor() {
        //System.out.println(cityState);
        //Geocoder geocoderForNullCoordinates = new Geocoder(EstatisticasFragment.rootView.getContext());
        //List<Address> enderecosForNullCoordinates;
        try {
            //enderecosForNullCoordinates = geocoderForNullCoordinates.getFromLocationName(cityState, 1);

            //if (enderecosForNullCoordinates.size() > 0) {
                //latitude = enderecosForNullCoordinates.get(0).getLatitude();
                //longitude = enderecosForNullCoordinates.get(0).getLongitude();
                //Log.v("tag", "coordenadas " + enderecosForNullCoordinates.get(0).getLatitude() + ", " + enderecosForNullCoordinates.get(0).getLongitude());

                System.out.println(">>>>>> Latitude = " + latitude + "\n>>>>>> Longitude = " + longitude);

                Geocoder geocoder;
                List<Address> addresses;
                geocoder = new Geocoder(EstatisticasFragment.rootView.getContext(), Locale.getDefault());

                addresses = geocoder.getFromLocation(latitude, longitude, 1);
                postalCode = addresses.get(0).getPostalCode();
                String addressLine = addresses.get(0).getAddressLine(0);
                //System.out.println(">>>>>> Endereço = " + addressLine);
                if (postalCode != null && postalCode.length() == 5)
                    postalCode = postalCode + "-000";
                //System.out.println(">>>>>> CEP = " + postalCode);
            //}

        } catch (IOException ex) {
            if (ex != null) {
                ex.printStackTrace();
            } else {
                ex.printStackTrace();
            }
        }
    }

    private static void setarOMaximoDeResultadosDuranteOCarregamentoDosRecords() {
        EstatisticasFragment.ef.loadingIndicator.setMax(acumuladorResultados);
        ResultadosFragment.rf.loadingIndicatorResultados.setMax(acumuladorResultados);
        DeletadosFragment.df.loadingIndicatorDeletados.setMax(acumuladorResultados);
    }

    private static int filtrarTodosOsResultadosApenasParaSetarOMaximoDeResultadosDuranteOCarregamentoDosRecords(JSONArray recordArray) throws JSONException {
        System.out.println(recordArray.length());

        for (int i = 0; i < recordArray.length(); i++) {
            JSONObject currentResult = recordArray.getJSONObject(i);
            String title = currentResult.getString("title");
            String condition = currentResult.getString("condition");

            if (MainActivity.mn.filtrarResultadosJSON(title, condition)) {
                acumuladorResultados++;
            }

        }
        return acumuladorResultados;
    }

    static String trimPrice(String media) {
        media = media.replace(",", ".");
        double mediaInt = Double.parseDouble(media);
        return String.format("%.2f", mediaInt);
    }

    static public void parse(InputStream in) throws XmlPullParserException, IOException {
        try {
            XmlPullParser parser = Xml.newPullParser();
            parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
            parser.setInput(in, null);
            parser.nextTag();
            readFeed(parser);
        } finally {
            in.close();
        }
    }


    static private void readFeed(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, ns, "Servicos");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String name = parser.getName();
            // Começa pela entry tag
            if (name.equals("cServico")) {
                readEntry(parser);
            } else {
                skip(parser);
            }
        }
    }

    //Realiza o parse do XML retornado pelo correio
        static private void readEntry(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, ns, "cServico");
        String codigo = null;
        String valor;
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String name = parser.getName();
            if (name.equals("Codigo")) {
                codigo = readCodigo(parser);
                System.out.println(codigo);
            } else if (name.equals("Valor")) {
                valor = readValor(parser);
                System.out.println(valor);
                if (codigo.equals("04014")) {
                    String precoSedexString = valor;
                    precoSedex = Double.parseDouble(precoSedexString.replace(",", "."));
                } else if (codigo.equals("04510")) {
                    String precoPacString = valor;
                    precoPac = Double.parseDouble(precoPacString.replace(",", "."));
                }
            } else if (name.equals("Erro")){
                tagErro = readErro(parser);
                System.out.println("tagErro em QueryUtils = " + tagErro);
            } else {
                skip(parser);
            }
        }
    }

    static private String readCodigo(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "Codigo");
        String codigo = readText(parser);
        parser.require(XmlPullParser.END_TAG, ns, "Codigo");
        return codigo;
    }

    static private String readValor(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "Valor");
        String valor = readText(parser);
        parser.require(XmlPullParser.END_TAG, ns, "Valor");
        return valor;
    }

    static private String readErro(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "Erro");
        String valor = readText(parser);
        parser.require(XmlPullParser.END_TAG, ns, "Erro");
        return valor;
    }

    static private String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
        String result = "";
        if (parser.next() == XmlPullParser.TEXT) {
            result = parser.getText();
            parser.nextTag();
        }
        return result;
    }

    static private void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
        if (parser.getEventType() != XmlPullParser.START_TAG) {
            throw new IllegalStateException();
        }
        int depth = 1;
        while (depth != 0) {
            switch (parser.next()) {
                case XmlPullParser.END_TAG:
                    depth--;
                    break;
                case XmlPullParser.START_TAG:
                    depth++;
                    break;
            }
        }
    }

    private static String extractFeatureFromSellerJson(String recordJSON) {

        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        String sellerReputation = null;

        try {
            JSONObject baseJsonResponse = new JSONObject(recordJSON);

            JSONObject sellerReputationJsonResponse = baseJsonResponse.getJSONObject("seller_reputation");
            sellerReputation = sellerReputationJsonResponse.getString("level_id");

        } catch (JSONException e) {

            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }

        }

        // Retorna a reputação
        return sellerReputation;
    }


    private static String[] extractFeatureFromMercadoEnviosJson(String recordJSON) {

        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        String[] valoresMercadoLivre = new String[2];

        try {
            JSONObject baseJsonResponse = new JSONObject(recordJSON);
            JSONArray optionsJsonResponse = baseJsonResponse.getJSONArray("options");
            JSONObject mercadoEnviosExpressoJSONObject = optionsJsonResponse.getJSONObject(0);
            valoresMercadoLivre[0] = mercadoEnviosExpressoJSONObject.getString("cost");

            JSONObject mercadoEnviosNormalJSONObject = optionsJsonResponse.getJSONObject(1);
            valoresMercadoLivre[1] = mercadoEnviosNormalJSONObject.getString("cost");
        } catch (JSONException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        }

        // Returna valores do Mercado Envios
        return valoresMercadoLivre;
    }

    private static String extractFeatureFromCustomPriceJson(String recordJSON) {
        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        String valorCustom = null;

        try {

            JSONObject baseJsonResponse = new JSONObject(recordJSON);
            JSONArray optionsJsonResponse = baseJsonResponse.getJSONArray("options");
            JSONObject mercadoEnviosExpressoJSONObject = optionsJsonResponse.getJSONObject(0);
            valorCustom = mercadoEnviosExpressoJSONObject.getString("cost");


        } catch (JSONException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        }

        // Retorna o valor de preço Tabelado
        return valorCustom;
    }

    private static String extractFeatureFromCityJson(String recordJSON) {
        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        String cepFromCity = null;

        try {

            JSONObject baseJsonResponse = new JSONObject(recordJSON);
            cepFromCity = baseJsonResponse.getString("cep");

        } catch (JSONException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        }

        // Retorna o CEP
        return cepFromCity;
    }

    private static Date extractFeatureFromDataJson(String recordJSON) {
        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        Date anuncioId = null;
        String anuncioIdNaoFormatado;

        try {

            JSONObject baseJsonResponse = new JSONObject(recordJSON);
            anuncioIdNaoFormatado = baseJsonResponse.getString("start_time");

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
            anuncioId = sdf.parse(anuncioIdNaoFormatado);

            JSONArray picturesArray = baseJsonResponse.getJSONArray("pictures");
                JSONObject picturesFirstIndex = picturesArray.getJSONObject(0);
            urlPictureBig = picturesFirstIndex.getString("url");
            urlPictureBigSize = picturesFirstIndex.getString("size");

            JSONObject geolocationObject = baseJsonResponse.getJSONObject("geolocation");
            latitude = geolocationObject.getDouble("latitude");
            longitude = geolocationObject.getDouble("longitude");

            System.out.println("latlong = " + latitude + " " + longitude);
            utilizarGeocoderParaDecifrarCepDoVendedor();

        } catch (JSONException e) {

            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }

        // Retorna o JSON
        return anuncioId;
    }

    private static String extractFeatureFromPictureJson(String recordJSON) {
        if (TextUtils.isEmpty(recordJSON)) {

            return null;

        }

        String urlDeImagemEmAltaQualidadeEmString = null;

        try {

            JSONObject baseJsonResponse = new JSONObject(recordJSON);

            JSONArray recordArray = baseJsonResponse.getJSONArray("pictures");
            JSONObject urlDeImagemEmAltaQualidadeEmJSONObject = recordArray.getJSONObject(0);

            urlDeImagemEmAltaQualidadeEmString = urlDeImagemEmAltaQualidadeEmJSONObject.getString("url");

        } catch (JSONException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        }

        return urlDeImagemEmAltaQualidadeEmString;
    }

    public static String fetchViaCEP(String requestUrl) {
        return extractFeatureFromViaCEPJson(instanciarUrlEFazerRequestHttp(requestUrl));
    }
    private static String extractFeatureFromViaCEPJson(String recordJSON) {
        if (TextUtils.isEmpty(recordJSON)) {
            return null;
        }
        String erro = null;
        try {
            JSONObject baseJsonResponse = new JSONObject(recordJSON);
            erro = baseJsonResponse.getString("erro");
        } catch (JSONException e) {
            if (e != null) {
                e.printStackTrace();
            } else {
                Log.e("QueryUtils", "Problema realizando parse do JSON", e);
            }
        }

        // Return the list of records
        return erro;
    }


    public static boolean isCEPValido(final String cep) {

        Runnable runnableParaPegarInformacoesDosCorreios = new Runnable() {
            public void run() {
                    Uri baseUri = Uri.parse(VIACEP_REQUEST_URL);
                    Uri.Builder uriBuilder = baseUri.buildUpon();
                    uriBuilder.appendPath(cep);
                    uriBuilder.appendPath("json");
                    CodigoDeErro = fetchViaCEP(uriBuilder.toString());
            }
        };

        threadParaPegarInformacoesDosCorreios = new Thread(runnableParaPegarInformacoesDosCorreios);
        threadParaPegarInformacoesDosCorreios.start();

        try {
            threadParaPegarInformacoesDosCorreios.join();
            System.out.println("CodigoDeErro = " + CodigoDeErro);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        if(CodigoDeErro != null && CodigoDeErro.equals("true")){
            return false;
        } else {
            return true;
        }
    }
}