/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.example.android.vinylfinder;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.util.SparseBooleanArray;
import android.view.ActionMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;

/**
 * Fragmento utilizado para aba Resultados
 */
public class ResultadosFragment extends Fragment {
    static public String cepUsuario;
    static ResultadosFragment rf;
    public ProgressBar loadingIndicatorResultados;
    public TextView textViewLoadingIndicatorResultados;
    //public static boolean intentAberto = false;

    /**
     * Adapter para List de Records
     */
    ResultadosRecordAdapter mAdapter;

    static List<Record> recordsGlobal = new ArrayList<>();
    static public Menu mOptionsMenu;
    static View rootView;

    boolean removerUsuariosSemReputacao;

    public Context resultadosFragmentContext;
    public Activity activityFragmentContext;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
        rf = ResultadosFragment.this;
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

        resultadosFragmentContext = getContext();
        rootView = inflater.inflate(R.layout.fragment_resultados, container, false);

        setRetainInstance(true);

        //Limpar a tela
        esconderTeclado(this.getActivity());
        esconderLoadingIndicator();
        esconderTextViewDeAndamentoDoLoadingIndicator();
        
        configurarListView();

        setHasOptionsMenu(true);

        return rootView;

    }

    //Realiza a pré-configuração da ListView, criando listeners e associando campos com os ids presentes em Resources
    private void configurarListView() {
        setarTextoASerExibidoParaListViewVazia((ListView) rootView.findViewById(R.id.listview_record));
        setarAdaptadorParaListView((ListView) rootView.findViewById(R.id.listview_record));
        setarModoDeEscolhaParaDelecoesMultiplasNaListView((ListView) rootView.findViewById(R.id.listview_record));
        setarClickListenerParaListView((ListView) rootView.findViewById(R.id.listview_record));
    }

    private void setarClickListenerParaListView(ListView recordListView) {
        recordListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long l) {
                // Retorna o Record que foi clicado
                Record recordSendoProcessadoNoMomento = mAdapter.getItem(position);
                // Converte o URL String em um objeto URI (para passar no construtor de Intent)
                Uri recordUri = Uri.parse(recordSendoProcessadoNoMomento.getmPermalink());
                Intent intent = new Intent(Intent.ACTION_VIEW, recordUri);
                startActivity(intent);
                //intentAberto = true;
                //System.out.println(intentAberto);
            }
        });
    }


    private void setarModoDeEscolhaParaDelecoesMultiplasNaListView(final ListView recordListView) {
        recordListView.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE_MODAL);
        recordListView.setMultiChoiceModeListener(new AbsListView.MultiChoiceModeListener() {

            @Override
            public boolean onPrepareActionMode(android.view.ActionMode mode, Menu menu) {
                // TODO  Auto-generated method stub
                return false;
            }


            @Override
            public void onDestroyActionMode(android.view.ActionMode mode) {
                // TODO  Auto-generated method stub

            }

            @Override
            public boolean onCreateActionMode(android.view.ActionMode mode, Menu menu) {
                // TODO  Auto-generated method stub
                mode.getMenuInflater().inflate(R.menu.menu_resultados_contextual, menu);
                return true;

            }

            @Override
            public boolean onActionItemClicked(final android.view.ActionMode mode,
                                               MenuItem item) {
                // TODO  Auto-generated method stub

                switch (item.getItemId()) {
                    case R.id.selectAll:
                        removerItensPossivelmenteJaSelecionados();
                        selecionarTodosOsItemsDaListView((ListView) rootView.findViewById(R.id.listview_record));
                        alterarOTituloDaActionBarParaRefletirOTotalDeItensSelecionados(mode);
                        return true;
                    case R.id.delete:
                        //Deleção em sí vai acontecer na chamada do método abaixo
                        AlertDialog.Builder builder = configurarAlertDialogDeConfirmacaoDeDelecaoDeItens(mode);
                        AlertDialog alert = builder.create();
                        //alert.setIcon(R.drawable.questionicon);// dialog  Icon
                        alert.setTitle("Confirmação"); // dialog  Title
                        alert.show();
                        return true;
                    default:
                        return false;
                }

            }

            @Override
            public void onItemCheckedStateChanged(android.view.ActionMode mode,
                                                  int position, long id, boolean checked) {
                // TODO  Auto-generated method stub
                final int checkedCount = recordListView.getCheckedItemCount();
                mode.setTitle(checkedCount + "  Selecionado(s)");
                mAdapter.toggleSelection(position);
            }
        });
    }

    private AlertDialog.Builder configurarAlertDialogDeConfirmacaoDeDelecaoDeItens(ActionMode mode) {
        AlertDialog.Builder builder = new AlertDialog.Builder(
                getContext());
        builder.setMessage("Confirmar exclusão do(s) disco(s) selecionado(s)?");

        configurarBotaoNegativo(builder);
        //Caso positivo, será deletado através do método abaixo
        configurarBotaoPositivo(builder, mode);
        
        return builder;
    }

    private void configurarBotaoPositivo(AlertDialog.Builder builder, final ActionMode mode) {
        builder.setPositiveButton("Sim", new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                // TODO  Auto-generated method stub
                SparseBooleanArray selected = mAdapter
                        .getSelectedIds();
                for (int i = (selected.size() - 1); i >= 0; i--) {
                    if (selected.valueAt(i)) {
                        Record itemSelecionado = mAdapter
                                .getItem(selected.keyAt(i));

                        //Adiciona os itens na aba Deletados
                        adicionarItemEmDeletadosFragment(itemSelecionado);
                        //Remove os itens da aba Resultados
                        removerItemDeResultadosFragment(itemSelecionado);
                        //Reconfigura a média final, na aba Estatísticas
                        configurarTextViewMediaFinal(realizarSomaDosPrecos());
                        //configurarTextViewSomaDosRecordsDesvioPadraoECoeficienteDeVariacao(realizarSomaDosPrecos());

                    }
                }
                mode.finish();
                selected.clear();
                notificarMudancasNoAdaptadorEmDeletadosFragment();
                notificarMudancasNoAdaptadorEmResultadosFragment();

            }
        });
    }

    private void notificarMudancasNoAdaptadorEmResultadosFragment() {
        mAdapter.notifyDataSetChanged();
        mAdapter.notifyDataSetInvalidated();
    }

    private void notificarMudancasNoAdaptadorEmDeletadosFragment() {
        DeletadosFragment.df.mDeletadosAdapter.notifyDataSetChanged();
        DeletadosFragment.df.mDeletadosAdapter.notifyDataSetInvalidated();
    }

    private void configurarTextoDaTextView(TextView totalDeRegistros, String somaDosRecordsEmString, Double desvioPadrao, Double coeficienteVariacao) {
        totalDeRegistros.setText("Total de Registros: " + somaDosRecordsEmString + "\nDesvio Padrão: " + desvioPadrao + "\nC.V: " + coeficienteVariacao);
        totalDeRegistros.setTextSize(10);
        totalDeRegistros.setTextColor(ContextCompat.getColor(EstatisticasFragment.rootView.getContext(), R.color.colorPrice));
    }

    private void configurarTextViewMediaFinal(Double somaDosPrecos) {
        Double mediaFinalEmDouble = calcularMediaFinalERetornarEmDouble(somaDosPrecos);
        TextView mediaFinal = (TextView) EstatisticasFragment.rootView.findViewById(R.id.textViewMedia);
        configurarTextoDaTextViewMediaFinal(mediaFinalEmDouble, mediaFinal);
        mediaFinal.setVisibility(View.VISIBLE);

    }

    private void configurarTextoDaTextViewMediaFinal(Double mediaFinalEmDouble, TextView mediaFinal) {
        //mediaFinalEmString = QueryUtils.trimPrice(mediaFinalEmString);
        Activity activity = (Activity) EstatisticasFragment.rootView.getContext();
        mediaFinal.setText(converterDoubleEmReais(mediaFinalEmDouble));
        mediaFinal.setTextSize(50);
        mediaFinal.setTextColor(ContextCompat.getColor(activity, R.color.colorAccent));
    }

    public String converterDoubleEmReais(Double d) {
        Locale ptBr = new Locale("pt", "BR");
        return NumberFormat.getCurrencyInstance(ptBr).format(d);
    }

    private Double calcularMediaFinalERetornarEmDouble(Double somaDosPrecos) {
        return calcularMediaEmDouble(somaDosPrecos);
    }

    private Double calcularMediaEmDouble(Double somaDosPrecos) {
        return EstatisticasFragment.fazerMedia(somaDosPrecos, ResultadosFragment.recordsGlobal.size());
    }

    private Double realizarSomaDosPrecos() {
            Double somaDosPrecos = 0.0;
            for (int j = 0; j < recordsGlobal.size(); j++) {
                somaDosPrecos += recordsGlobal.get(j).getmPrice();
                System.out.println("getmPrice(): " + recordsGlobal.get(j).getmPrice());
            }
            return somaDosPrecos;
        }

    private void removerItemDeResultadosFragment(Record itemSelecionado) {
        mAdapter.remove(itemSelecionado);
        recordsGlobal.remove(itemSelecionado);
    }

    private void adicionarItemEmDeletadosFragment(Record itemSelecionado) {
        DeletadosFragment.df.mDeletadosAdapter.add(itemSelecionado);
        DeletadosFragment.df.recordsDeletadosGlobal.add(itemSelecionado);
    }

    private void configurarBotaoNegativo(AlertDialog.Builder builder) {
        builder.setNegativeButton("Não", new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                // TODO  Auto-generated method stub
            }
        });
    }

    private void removerItensPossivelmenteJaSelecionados() {
        mAdapter.removeSelection();
    }

    private void selecionarTodosOsItemsDaListView(ListView recordListView) {
        for (int i = 0; i < recordsGlobal.size(); i++) {
            recordListView.setItemChecked(i, true);
            //  listviewadapter.toggleSelection(i);
        }
    }

    private void alterarOTituloDaActionBarParaRefletirOTotalDeItensSelecionados(android.view.ActionMode mode) {
        mode.setTitle(recordsGlobal.size() + "  Selecionados");
    }

    private void setarAdaptadorParaListView(ListView recordListView) {
        mAdapter = new ResultadosRecordAdapter(getActivity(), new ArrayList<Record>());
        recordListView.setAdapter(mAdapter);
    }

    private void esconderTextViewDeAndamentoDoLoadingIndicator() {
        textViewLoadingIndicatorResultados = (TextView) rootView.findViewById(R.id.textView_loading_indicator_resultados);
        textViewLoadingIndicatorResultados.setVisibility(View.GONE);
    }

    private void esconderLoadingIndicator() {
        loadingIndicatorResultados = (ProgressBar) rootView.findViewById(R.id.loading_indicator_resultados);
        loadingIndicatorResultados.setVisibility(View.GONE);
    }

    private void setarTextoASerExibidoParaListViewVazia(ListView recordsResultadosListView) {
        TextView mEmptyStateTextView = (TextView) rootView.findViewById(R.id.empty_view);
        recordsResultadosListView.setEmptyView(mEmptyStateTextView);
    }


    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.menu_resultados, menu);
        super.onCreateOptionsMenu(menu, inflater);
        mOptionsMenu = menu;
    }

    private FloatingActionButton getFAB() {
        FloatingActionButton fab = new FloatingActionButton(getContext());
        fab.setBackgroundDrawable(ContextCompat.getDrawable(getContext(), R.drawable.ic_filter_list_black_24dp));
        return fab;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if (item.isChecked())
            item.setChecked(false);
        else
            item.setChecked(true);

        switch (item.getItemId()) {
            case R.id.menuRemoverAnunciosPorReputacaoDoVendedor:
                exibirMensagemReputacaoMaxima();
                return true;
                /**case R.id.menuSortClosest:
                exibirMensagemEdt();
                return true;**/
            case R.id.menuDeleteMaiorQue:
                exibirMensagemDeletarMaiorQue();
                return true;
            case R.id.menuSortPrecoMenorNormalPacTabelado:
                sortCheaperNormalPacTabelado();
                limparReadicionarENotificarMudancasAoMAdapter();
                return true;
            case R.id.menuSortPrecoMenorExpressoSedex:
                sortCheaperExpressoSedexTabelado();
                limparReadicionarENotificarMudancasAoMAdapter();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }

    }

    private void limparReadicionarENotificarMudancasAoMAdapter() {
        mAdapter.clear();
        mAdapter.addAll(recordsGlobal);
        mAdapter.notifyDataSetChanged();
        mAdapter.notifyDataSetInvalidated();
    }

    public static void esconderTeclado(Activity activity) {
        InputMethodManager inputMethodManager =
                (InputMethodManager) activity.getSystemService(
                        Activity.INPUT_METHOD_SERVICE);
        inputMethodManager.hideSoftInputFromWindow(
                new View(activity).getWindowToken(), 0);
        inputMethodManager.hideSoftInputFromWindow(
                new View(activity).getWindowToken(), 0);
    }

    //Popup para inserção do preço máximo
    public void exibirMensagemDeletarMaiorQue() {
        final EditText input = new EditText(getContext());
        input.setRawInputType(Configuration.KEYBOARD_12KEY);
        final AlertDialog dialog = new AlertDialog.Builder(getContext())
                .setView(rootView)
                .setTitle("Inserir Preço Máximo:")
                .setView(input)
                .setPositiveButton(android.R.string.ok, null)
                .create();

        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(final DialogInterface dialog) {

                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {
                        String inputString = input.getText().toString();
                        //Conversão de String para Double
                        Double inputDouble = Double.parseDouble(inputString);
                        //For para percorrer o vetor que contem todos os Records
                        for (int i = 0; i < recordsGlobal.size(); i++) {
                            //Se o preço do anúncio for maior que o definido pelo usuário...
                            if (recordsGlobal.get(i).getmPrice() > inputDouble) {
                                System.out.println(">>> Entrei no IF de preço maior oque o estipulado pelo usuário");
                                Record recordSendoProcessadoNoMomento = mAdapter.getItem(i);
                                //...Adicioná-lo na DeletadosFragment
                                adicionarRecordEmDeletadosFragment(i, recordSendoProcessadoNoMomento);
                                //...Removê-lo da ResultadosFragment
                                removerRecordDeResultadosFragment(i, recordSendoProcessadoNoMomento);
                                //Atualizar a média em EstatisticasFragment
                                configurarTextViewMediaFinal(realizarSomaDosPrecos());
                                //configurarTextViewSomaDosRecordsDesvioPadraoECoeficienteDeVariacao(realizarSomaDosPrecos());
                                i--;
                            }
                        }
                        dialog.dismiss();
                        mAdapter.notifyDataSetChanged();
                        esconderTeclado(getActivity());
                    }
                });
            }
        });

        dialog.show();
    }



    private void removerRecordDeResultadosFragment(int i, Record recordSendoProcessadoNoMomento) {
        recordsGlobal.remove(i);
        mAdapter.remove(recordSendoProcessadoNoMomento);
    }

    private void adicionarRecordEmDeletadosFragment(int i, Record recordSendoProcessadoNoMomento) {
        DeletadosFragment.df.recordsDeletadosGlobal.add(recordsGlobal.get(i));
        DeletadosFragment.df.mDeletadosAdapter.add(recordSendoProcessadoNoMomento);
    }

    //Popup para inserção da reputação
    public void exibirMensagemReputacaoMaxima() {

        View checkBoxView = View.inflate(this.getContext(), R.layout.checkbox, null);
        CheckBox checkBox = (CheckBox) checkBoxView.findViewById(R.id.checkbox);
        checkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {

            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                removerUsuariosSemReputacao = true;
            }
        });
        checkBox.setText("Deletar usuários sem reputação");

        final EditText input = (EditText) checkBoxView.findViewById(R.id.reputacao_maxima);
        input.setRawInputType(Configuration.KEYBOARD_12KEY);
        final AlertDialog dialog = new AlertDialog.Builder(getContext())
                .setView(rootView)
                .setTitle("Inserir Reputação Mínima")
                .setView(checkBoxView)
                .setPositiveButton(android.R.string.ok, null)
                .create();

        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(final DialogInterface dialog) {

                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {

                        String inputString = input.getText().toString();
                        int inputInt = Integer.parseInt(inputString);

                        dialog.dismiss();
                        editarRecordList(inputInt);
                        mAdapter.clear();
                        mAdapter.addAll(recordsGlobal);
                        mAdapter.notifyDataSetChanged();
                        mAdapter.notifyDataSetInvalidated();
                        esconderTeclado(getActivity());
                    }
                });
            }
        });

        dialog.show();
    }

    public void sortCheaperNormalPacTabelado() {
        Collections.sort(recordsGlobal, new Comparator<Record>() {
            public int compare(Record one, Record two) {
                if (one.getmPrecoPacOuNormalouTabelado() == null || two.getmPrecoPacOuNormalouTabelado() == null)
                    return 0;
                return one.getmPrecoPacOuNormalouTabelado().compareTo(two.getmPrecoPacOuNormalouTabelado());
            }
        });
    }

    public void sortCheaperExpressoSedexTabelado() {
        Collections.sort(recordsGlobal, new Comparator<Record>() {
            public int compare(Record one, Record two) {
                if (one.getmPrecoSedexOuExpresso() == null || two.getmPrecoSedexOuExpresso() == null)
                    return 0;
                return one.getmPrecoSedexOuExpresso().compareTo(two.getmPrecoSedexOuExpresso());
            }
        });
    }

    public void editarRecordList(int valorMaximoDeReputacao) {

        //Percorre todo o vetor de records
        for (int i = 0; i < recordsGlobal.size(); i++) {
            System.out.println("recordsGlobal " + recordsGlobal.size());
            System.out.println("iteração " + i);

            //Armazena o valor da reputação do record
            int recordSendoProcessadoNoMomentoReputation;
            System.out.println(recordsGlobal.get(i).getmPermalink());
            switch (recordsGlobal.get(i).getmSellerReputation()) {
                case "1_red":
                    recordSendoProcessadoNoMomentoReputation = 1;
                    break;
                case "2_orange":
                    recordSendoProcessadoNoMomentoReputation = 2;
                    break;
                case "3_yellow":
                    recordSendoProcessadoNoMomentoReputation = 3;
                    break;
                case "4_light_green":
                    recordSendoProcessadoNoMomentoReputation = 4;
                    break;
                case "5_green":
                    recordSendoProcessadoNoMomentoReputation = 5;
                    break;
                default:
                    recordSendoProcessadoNoMomentoReputation = 6;

            }

            //Se a flag de usuario sem reputação for selecionada, insere 0 no valor da reputação
            if (removerUsuariosSemReputacao){
                System.out.println(recordsGlobal.get(i).getmSellerReputation());
                if (recordsGlobal.get(i).getmSellerReputation().equals("null")){
                    recordSendoProcessadoNoMomentoReputation = 0;
                }
            }

            //Deleção, de fato, dos anúncios de vendedores com reputação menor que a selecionada
            if (recordSendoProcessadoNoMomentoReputation < valorMaximoDeReputacao) {

                Record recordSendoProcessadoNoMomento = mAdapter.getItem(i);
                adicionarRecordEmDeletadosFragment(i, recordSendoProcessadoNoMomento);
                removerRecordDeResultadosFragment(i, recordSendoProcessadoNoMomento);

                configurarTextViewMediaFinal(realizarSomaDosPrecos());
                //configurarTextViewSomaDosRecordsDesvioPadraoECoeficienteDeVariacao(realizarSomaDosPrecos());

                i--;
            }

        }
    }



}
