package com.example.android.vinylfinder;

import android.app.Activity;
import android.content.Context;
import android.support.v4.content.ContextCompat;
import android.util.SparseBooleanArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import com.koushikdutta.urlimageviewhelper.UrlImageViewHelper;

public class ResultadosRecordAdapter extends ArrayAdapter<Record> {
    List<Record> DataList;
    private SparseBooleanArray mSelectedItemsIds;
    Context myContext;
    LayoutInflater inflater;

    public ResultadosRecordAdapter(Activity context, ArrayList<Record> records) {
        super(context, 0, records);
        mSelectedItemsIds = new SparseBooleanArray();
        myContext = context;
        DataList = records;
        inflater = LayoutInflater.from(context);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        // Verifica se a view está sendo usada, senão infla
        View listItemView = convertView;
        if (listItemView == null) {
            listItemView = LayoutInflater.from(getContext()).inflate(
                    R.layout.list_item, parent, false);
        }

        // Pega o objeto localizado na posição do toque na tela
        Record recordAtualmenteSendoProcessado = getItem(position);

        //Configura as textViews e ImageViews de cada Resultado da ListView
        configurarTextViewDeNomeDoRecord(recordAtualmenteSendoProcessado, listItemView);
        configurarImageViewDoRecord(recordAtualmenteSendoProcessado, listItemView);
        configurarCorDoTermometro(recordAtualmenteSendoProcessado, listItemView);
        configurarTextViewDePrecoDoRecord(recordAtualmenteSendoProcessado, listItemView);
        configurarTextViewDeCidadeEEstadoDoVendedor(recordAtualmenteSendoProcessado, listItemView);
        configurarImageViewDeFreteGratis(recordAtualmenteSendoProcessado, listItemView);
        configurarTextViewsDePrecosComFreteIncluso(recordAtualmenteSendoProcessado, listItemView);

        //Retorna o layout completo, para ser utilizado na ListView
        return listItemView;
    }

    //Configura quais ícones e valores serão exibidos, conforme a opção de frete contida em cada anúncio
    private void configurarTextViewsDePrecosComFreteIncluso(Record recordAtualmenteSendoProcessado, View listItemView) {

        switch(recordAtualmenteSendoProcessado.getmShippingMode()){
            case "custom":
                esconderTextViewDoPrecoDosCorreios(listItemView);
                esconderLinearLayoutDoPrecoDosCorreios(listItemView);
                esconderTextViewDoPrecoDoMercadoLivreEnvios(listItemView);
                esconderLinearLayoutDoPrecoDoMercadoLivreEnvios(listItemView);

                mostrarTextViewDePrecoTabelado(listItemView, recordAtualmenteSendoProcessado);
                mostrarLinearLayoutDePrecoTabelado(listItemView);
                break;
            case "not_specified":
                esconderTextViewDoPrecoDoMercadoLivreEnvios(listItemView);
                esconderLinearLayoutDoPrecoDoMercadoLivreEnvios(listItemView);
                esconderTextViewDePrecoTabelado(listItemView);
                esconderLinearLayoutDePrecoTabelado(listItemView);

                mostrarTextViewDePrecoDosCorreios(listItemView, recordAtualmenteSendoProcessado);
                mostrarLinearLayoutDePrecoDosCorreios(listItemView);
                break;
            case "me1":
            case "me2":
                esconderTextViewDoPrecoDosCorreios(listItemView);
                esconderLinearLayoutDoPrecoDosCorreios(listItemView);
                esconderTextViewDePrecoTabelado(listItemView);
                esconderLinearLayoutDePrecoTabelado(listItemView);

                mostrarTextViewDePrecoDoMercadoLivreEnvios(listItemView, recordAtualmenteSendoProcessado);
                mostrarLinearLayoutDePrecoDoMercadoLivreEnvios(listItemView);
                break;
            default:
                esconderTextViewDoPrecoDoMercadoLivreEnvios(listItemView);
                esconderLinearLayoutDoPrecoDoMercadoLivreEnvios(listItemView);
                esconderTextViewDoPrecoDosCorreios(listItemView);
                esconderLinearLayoutDoPrecoDosCorreios(listItemView);

                esconderTextViewDePrecoTabelado(listItemView);
                esconderLinearLayoutDePrecoTabelado(listItemView);
                break;
        }


    }

    //Mercado Livre Envios
    private void mostrarLinearLayoutDePrecoDoMercadoLivreEnvios(View listItemView) {
        LinearLayout linearLayoutPrecoComExpresso = (LinearLayout) listItemView.findViewById(R.id.preco_menvios_expresso_layout);
        LinearLayout linearLayoutPrecoComNormal = (LinearLayout) listItemView.findViewById(R.id.preco_menvios_normal_layout);
        linearLayoutPrecoComExpresso.setVisibility(View.VISIBLE);
        linearLayoutPrecoComNormal.setVisibility(View.VISIBLE);
    }

    //Mercado Livre Envios
    private void mostrarTextViewDePrecoDoMercadoLivreEnvios(View listItemView, Record recordAtualmenteSendoProcessado) {
        TextView textViewPrecoComExpresso = (TextView) listItemView.findViewById(R.id.preco_com_menvios_expresso_text_view);
        TextView textViewPrecoComNormal = (TextView) listItemView.findViewById(R.id.preco_com_menvios_normal_text_view);
        textViewPrecoComExpresso.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoEnviosExpresso()) + "(" + converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoEnviosExpresso() + recordAtualmenteSendoProcessado.getmPrice()) + ")");
        textViewPrecoComNormal.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoEnviosNormal()) + "(" + converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoEnviosNormal() + recordAtualmenteSendoProcessado.getmPrice()) + ")");
    }

    //Correios
    private void mostrarLinearLayoutDePrecoDosCorreios(View listItemView) {
        LinearLayout linearLayoutPrecoComSedex = (LinearLayout) listItemView.findViewById(R.id.preco_sedex_layout);
        LinearLayout linearLayoutPrecoComPac = (LinearLayout) listItemView.findViewById(R.id.preco_pac_layout);
        linearLayoutPrecoComSedex.setVisibility(View.VISIBLE);
        linearLayoutPrecoComPac.setVisibility(View.VISIBLE);
    }

    //Correios
    private void mostrarTextViewDePrecoDosCorreios(View listItemView, Record recordAtualmenteSendoProcessado) {
        TextView textViewPrecoComSedex = (TextView) listItemView.findViewById(R.id.preco_com_sedex_text_view);
        TextView textViewPrecoComPac = (TextView) listItemView.findViewById(R.id.preco_com_pac_text_view);
        textViewPrecoComSedex.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmPrecoSedex()) + "(" + converterDoubleEmReais(recordAtualmenteSendoProcessado.getmPrecoSedex() + recordAtualmenteSendoProcessado.getmPrice()) + ")");
        textViewPrecoComPac.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmPrecoPac()) + "(" + converterDoubleEmReais(recordAtualmenteSendoProcessado.getmPrecoPac() + recordAtualmenteSendoProcessado.getmPrice()) + ")");
    }

    //Custom (Tabelado)
    private void esconderLinearLayoutDePrecoTabelado(View listItemView) {
        LinearLayout linearLayoutPrecoComTabela = (LinearLayout) listItemView.findViewById(R.id.preco_tabela_layout);
        linearLayoutPrecoComTabela.setVisibility(View.GONE);
    }

    //Custom (Tabelado)
    private void esconderTextViewDePrecoTabelado(View listItemView) {
        TextView textViewPrecoComTabela = (TextView) listItemView.findViewById(R.id.preco_com_tabela);
        textViewPrecoComTabela.setText("");
    }

    //Custom (Tabelado)
    private void mostrarLinearLayoutDePrecoTabelado(View listItemView) {
        LinearLayout linearLayoutPrecoComTabela = (LinearLayout) listItemView.findViewById(R.id.preco_tabela_layout);
        linearLayoutPrecoComTabela.setVisibility(View.VISIBLE);
    }

    //Custom (Tabelado)
    private void mostrarTextViewDePrecoTabelado(View listItemView, Record recordAtualmenteSendoProcessado) {
        TextView textViewPrecoComTabela = (TextView) listItemView.findViewById(R.id.preco_com_tabela);
        textViewPrecoComTabela.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoLivreCustomPrice()) + "(" + converterDoubleEmReais(recordAtualmenteSendoProcessado.getmMercadoLivreCustomPrice() + recordAtualmenteSendoProcessado.getmPrice()) + ")");

    }

    //Mercado Livre Envios
    private void esconderLinearLayoutDoPrecoDoMercadoLivreEnvios(View listItemView) {
        LinearLayout linearLayoutPrecoComExpresso = (LinearLayout) listItemView.findViewById(R.id.preco_menvios_expresso_layout);
        LinearLayout linearLayoutPrecoComNormal = (LinearLayout) listItemView.findViewById(R.id.preco_menvios_normal_layout);
        linearLayoutPrecoComExpresso.setVisibility(View.GONE);
        linearLayoutPrecoComNormal.setVisibility(View.GONE);
    }

    //Mercado Livre Envios
    private void esconderTextViewDoPrecoDoMercadoLivreEnvios(View listItemView) {
        TextView textViewPrecoComExpresso = (TextView) listItemView.findViewById(R.id.preco_com_menvios_expresso_text_view);
        TextView textViewPrecoComNormal = (TextView) listItemView.findViewById(R.id.preco_com_menvios_normal_text_view);
        textViewPrecoComExpresso.setText("");
        textViewPrecoComNormal.setText("");
    }

    //Correios
    private void esconderLinearLayoutDoPrecoDosCorreios(View listItemView) {
        LinearLayout linearLayoutPrecoComSedex = (LinearLayout) listItemView.findViewById(R.id.preco_sedex_layout);
        LinearLayout linearLayoutPrecoComPac = (LinearLayout) listItemView.findViewById(R.id.preco_pac_layout);
        linearLayoutPrecoComSedex.setVisibility(View.GONE);
        linearLayoutPrecoComPac.setVisibility(View.GONE);
    }

    //Correios
    private void esconderTextViewDoPrecoDosCorreios(View listItemView) {
        TextView textViewPrecoComSedex = (TextView) listItemView.findViewById(R.id.preco_com_sedex_text_view);
        TextView textViewPrecoComPac = (TextView) listItemView.findViewById(R.id.preco_com_pac_text_view);
        textViewPrecoComPac.setText("");
        textViewPrecoComSedex.setText("");
    }

    //Imagem de Frete Grátis (Caminhão)
    private void configurarImageViewDeFreteGratis(Record recordAtualmenteSendoProcessado, View listItemView) {
        ImageView freeShippingImageView = (ImageView) listItemView.findViewById(R.id.imageViewFreteGratis);
        if (recordAtualmenteSendoProcessado.ismFreeShipping()) {
            freeShippingImageView.setVisibility(View.VISIBLE);
        } else {
            freeShippingImageView.setVisibility(View.GONE);
        }
    }

    //Cidade - Estado
    private void configurarTextViewDeCidadeEEstadoDoVendedor(Record recordAtualmenteSendoProcessado, View listItemView) {
        TextView sellerLocationTextView = (TextView) listItemView.findViewById(R.id.seller_location);
        sellerLocationTextView.setText("(" + recordAtualmenteSendoProcessado.getmSellerLocation() + ")");
    }

    //Preço em vermelho, que aparece logo abaixo do thumbnail
    private void configurarTextViewDePrecoDoRecord(Record recordAtualmenteSendoProcessado, View listItemView) {
        TextView numberTextView = (TextView) listItemView.findViewById(R.id.record_price);
        numberTextView.setText(converterDoubleEmReais(recordAtualmenteSendoProcessado.getmPrice()));
        numberTextView.setTextSize(20);
        numberTextView.setTextColor(ContextCompat.getColor(getContext(), R.color.colorPrice));
    }

    //Configura cor referente à reputação do vendedor
    private void configurarCorDoTermometro(Record recordAtualmenteSendoProcessado, View listItemView) {
        View termometherIndicatorView = (View) listItemView.findViewById(R.id.termometherIndicator);

        switch (recordAtualmenteSendoProcessado.getmSellerReputation()) { // ta dando problema de nullPointer aleatóriamente
            case "1_red":
                termometherIndicatorView.setBackgroundColor(0xFFF4111C);
                break;
            case "2_orange":
                termometherIndicatorView.setBackgroundColor(0xFFFF8000);
                break;
            case "3_yellow":
                termometherIndicatorView.setBackgroundColor(0xFFFFF059);
                break;
            case "4_light_green":
                termometherIndicatorView.setBackgroundColor(0xFFD6F076);
                break;
            case "5_green":
                termometherIndicatorView.setBackgroundColor(0xFF00B200);
                break;
            default:
                termometherIndicatorView.setBackgroundColor(0xFFF4F4F4);

        }
    }

    //Configura thumbnail do disco
    private void configurarImageViewDoRecord(Record recordAtualmenteSendoProcessado, View listItemView) {
        ImageView imageViewMiniatura = (ImageView) listItemView.findViewById(R.id.imageViewMiniatura);
        UrlImageViewHelper.setUrlDrawable(imageViewMiniatura, recordAtualmenteSendoProcessado.getmThumbnailUrl());
    }

    //Configura nome do disco
    private void configurarTextViewDeNomeDoRecord(Record recordAtualmenteSendoProcessado, View listItemView) {
        TextView TextViewDeNomeDoRecord = (TextView) listItemView.findViewById(R.id.record_name);

        TextViewDeNomeDoRecord.setText(recordAtualmenteSendoProcessado.getmName());
        TextViewDeNomeDoRecord.setTextSize(15);
        TextViewDeNomeDoRecord.setTextColor(ContextCompat.getColor(getContext(), R.color.colorTitle));
    }

    @Override
    public void remove(Record object) {
        DataList.remove(object);
        notifyDataSetChanged();
    }

    public String converterDoubleEmReais(Double d) {
        Locale ptBr = new Locale("pt", "BR");
        return NumberFormat.getCurrencyInstance(ptBr).format(d);
    }

    public List<Record> getMyList() {
        return DataList;
    }

    public void toggleSelection(int position) {
        selectView(position, !mSelectedItemsIds.get(position));
    }

    // Remove seleção depois do uncheck
    public void removeSelection() {
        mSelectedItemsIds = new SparseBooleanArray();
        notifyDataSetChanged();
    }

    // Itens selecionados
    public void selectView(int position, boolean value) {
        if (value)
            mSelectedItemsIds.put(position, value);
        else
            mSelectedItemsIds.delete(position);
        notifyDataSetChanged();
    }

    public int getSelectedCount() {
        return mSelectedItemsIds.size();
    }

    public SparseBooleanArray getSelectedIds() {
        return mSelectedItemsIds;
    }


}
